
Unit UTCParser;

Interface

Uses UTComplex, SysUtils, StrUtils;

{.$define UseAsm}
{$ASMMODE intel}

Type TCFunction = Procedure (Var C1, C2: TComplex) Of object;
  //Eine komplexe Rechenoperation als Funktion

Const Operators: array[0..4] Of Char = ('+', '-', '*', '/', '^');

Const Functions: array[0..6] Of string = ('abs', 'mod', 'sqr', 'ln', 'exp', 'neg', 'dbl');
  //Die Elemente, die im Parser-String auftauchen dürfen

Type 
  NodePointer = ^TCNode;
  //Ein Zeiger auf einen Ast im Baum
  TCNode = packed Record
    //Der Ast selber
    F: TCFunction;
    //Der Zeiger auf die Rechenoperation, die der Ast durchführt
    Par1, Par2: NodePointer;
    //Zeiger auf die Äste, die die Argumente liefern
    nmbrV, nmbrO, nmbrF: integer;
    //Nummer der Variable, des Operators, der Funktion
    Res: TComplex;
    //Zwischenspeicher für das Ergebnis von F, hält den Variablenwert
  End;

Type 
  TCParser = Class(TObject)
    Private 
      root: NodePointer;
      //Der Ursprung des Baumes
      Output,Input: string;
      //Output, Input-string

      OperatorFunctions: array[0..4] Of TCFunction;
      //enthält die Funktionen für Addition, Subtraktion...
      FunctionsFunctions: array[0..6] Of TCFunction;
      //Funktionen für abs(), mod(), sqr(), ln(), ...
      Variables: array Of string;
      //Namen der Variablen V1, V2, V3, ...
      VariablesValues: array Of TComplex;
      //Werte der Variablen V1=x+i*y, ...
      ZValues, CValues: array Of NodePointer;
      //Äste, die Z oder C zurückgeben sollen, ZValues[i]^.F=@ReturnZ

      //++++++++++Variablenverwaltung++++++++++
      Procedure AddV(C: TComplex; Var i: integer);
      Function FindV(s: String): integer;
      Procedure AddZ(N: NodePointer);
      Procedure AddC(N: NodePointer);
      Procedure UpdateZ(C: TComplex);
      Procedure UpdateC(C: TComplex);

      //++++++++++Baumverwaltung++++++++++
      Procedure SwitchNode(Var N1, N2: NodePointer);
      //N1 = N2; N2 = N1;
      Procedure ReplaceNode(Var N1, N2: NodePointer);
      //N1 = N2; Dispose(N2);
      Function CopyNode(N: NodePointer): NodePointer;
      //Result^ = N1^
      Procedure DisposeNode(N: NodePointer);
      //Dispose(N^.Par1, N^.Par2)

      //++++++++++Parsen++++++++++
      Function PrepareParsing: integer;
      //bereitet den Input-string fürs Parsen vor
      Function DoParsing(s: String): NodePointer;
      //parst den String s rekursiv und bildet einen Baum

      //++++++++++Mathematische Operationen am Baum++++++++++
      Procedure EvaluateNode(Var N: NodePointer);
      //wertet einen Ast aus
      Procedure DifferentiateNode(N: NodePointer);
      //differenziert einen Ast
      Procedure SimplifyNode(N: NodePointer);
      //vereinfacht einen Ast

      //++++++++++Helfer-Funktionen++++++++++
      Function GetOutput: string;
      //gibt den geparsten String aus
      Function FunctionToString(N: NodePointer; priority: integer): string;
      //wandelt einen geparsten Ast in einen Formel-String um
      Procedure PrintNode(N: NodePointer; depth: integer);
      //Einen Ast mit der Tiefe depth im Memo ausgeben (Debug-Output)

      //++++++++++komplexe Rechenoperationen++++++++++
      //analog zu den Funktionen in UTComplex
      Procedure PAdd(Var C1, C2: TComplex);
      Procedure PSub(Var C1, C2: TComplex);
      Procedure PMul(Var C1, C2: TComplex);
      Procedure PDiv(Var C1, C2: TComplex);
      Procedure PPower(Var C1, C2: TComplex);
      Procedure PAbs(Var C1, C2: TComplex);
      Procedure PMod(Var C1, C2: TComplex);
      Procedure PSqr(Var C1, C2: TComplex);
      Procedure PLn(Var C1, C2: TComplex);
      Procedure PExp(Var C1, C2: TComplex);
      Procedure PNeg(Var C1, C2: TComplex);
      Procedure PDouble(Var C1, C2: TComplex);
      Procedure ReturnZ(Var C1, C2: TComplex);
      Procedure ReturnC(Var C1, C2: TComplex);
    Public 
      constructor Create;
      ActiveZ, ActiveC: TComplex;
      //enthält die Variablen Z, C
      Function Parse(s: String): integer;
      //weist den Parser an, den string zu parsen und einen Baum aufzubauen
      Procedure Differentiate;
      //Differenziert über Z
      Procedure Simplify;
      //Vereinfacht
      Procedure PrintTree;
      //gibt den kompletten Baum in einem Memo aus
      Procedure ClearTree;
      //löscht den Baum ! (gibt den Speicher frei)
      Function Evaluate: TComplex;
      register;
      //Wertet den Baum aus
      property ParsedString: string read GetOutput;
      //gibt den geparsten String zurück
      property Tree: NodePointer read CopyTree write root;
      //gibt Zugriff auf die Wurzel des Baums (root)
  End;

Function IsOperator(p: char): integer;
Function IsFunction(s: String; pos: integer): boolean;

Implementation


{###########################################################################
 |                       Variablenverwaltung                               |
 ###########################################################################}
Procedure TCParser.AddV(C: TComplex; Var i: integer);

Var 
  found: boolean;
Begin
  i := -1;
  found := false;
  While i<High(VariablesValues) Do
    Begin
      inc(i);
      If VariablesValues[i]=C Then
        Begin
          found := true;
          break;
        End;
    End;
  If Not found Then
    Begin
      i := Length(VariablesValues)+1;
      SetLength(VariablesValues, i);
      SetLength(Variables, i);
      VariablesValues[i] := C;
      Variables[i] := 'V'+IntToStr(i);
    End;
End;
Function TCParser.FindV(s: String): integer;

Var 
  i: integer;
Begin
  i := 0;
  While i<=High(Variables) Do
    If Variables[i]=s Then break
    Else inc(i);
End;
Procedure TCParser.AddZ(N: NodePointer);

Var 
  i: integer;
  found: boolean;
Begin
  found := false;
  For i:=0 To High(ZValues) Do
    If N=@ZValues[i] Then
      Begin
        found := true;
        break;
      End;
  If Not found Then
    Begin
      SetLength(ZValues, Length(ZValues)+1);
      ZValues[High(ZValues)] := N;
    End;
End;
Procedure TCParser.AddC(N: NodePointer);

Var 
  i: integer;
  found: boolean;
Begin
  found := false;
  For i:=0 To High(CValues) Do
    If N=@CValues[i] Then
      Begin
        found := true;
        break;
      End;
  If Not found Then
    Begin
      SetLength(CValues, Length(CValues)+1);
      ZValues[High(CValues)] := N;
    End;
End;
Procedure TCParser.UpdateZ(C: TComplex);

Var 
  i: integer;
Begin
  For i:=0 To High(ZValues) Do
    ZValues[i]^.Res := C;
End;
Procedure TCParser.UpdateC(C: TComplex);

Var 
  i: integer;
Begin
  For i:=0 To High(CValues) Do
    CValues[i]^.Res := C;
End;


{###########################################################################
 |                          Baumverwaltung                                 |
 ###########################################################################}
Procedure TCParser.SwitchNode(Var N1, N2: NodePointer);

Var 
  tmp: NodePointer;
Begin
  tmp := N1;
  N1 := N2;
  N2 := tmp;
End;
Procedure TCParser.ReplaceNode(Var N1, N2: NodePointer);

Var 
  tmp: NodePointer;
Begin
  tmp := N1;
  N1 := N2;
  Dispose(N1);
End;
Function TCParser.CopyNode(N: NodePointer): NodePointer;
Begin
  If N=Nil Then Result := Nil
  Else
    Begin
      new(Result);
      Result^.F := N^.F;
      Result^.Res := N^.Res;
      Result^.Par1 := CopyNode(N^.Par1);
      Result^.Par2 := CopyNode(N^.Par2);
    End;
End;
Procedure TCParser.DisposeNode(N: NodePointer);
Begin
  If N = Nil Then exit;
  If N^.Par1 <> Nil Then DisposeNode(N^.Par1);
  If N^.Par1 <> Nil Then DisposeNode(N^.Par2);
  Dispose(N);
End;



{###########################################################################
 |                             Parsen                                      |
 ###########################################################################}
Function TCParser.PrepareParsing: integer;

Var 
  i, j: integer;
  OpenTokens: integer;
  startC, start2C, endC: integer;
  RPart, IPart: extended;
  RPartSet, IPartSet, useful: boolean;
Begin
  Output := Input;

  //Leerzeichen entfernen
  i := 1;
  While i <= Length(Output) Do
    Begin
      If Output[i] = ' ' Then Delete(Output, i, 1)
      Else inc(i);
    End;

  //unäres Minus durch Neg(x) ersetzen
  //wird in Simplify zu Neg(x)
  i := 1;
  While i <= Length(Output) Do
    Begin
      If (Output[i]='-') And
         ((i<Length(Output)) And Not (Output[i+1] In ['0'..'9'])) And
         ((i=1) Or ((i>1) And (Output[i-1] = '(')))
        Then
        Begin
          startC := i;
          OpenTokens := 0;
          While i<Length(Output) Do
            Begin
              inc(i);
              If (OpenTokens=0) And
                 ((IsOperator(Output[i])<Length(Operators)) Or (i=Length(Output))) Or
                 (Output[i]=')') Then break;
              If (Output[i]='(') Then inc(OpenTokens)
              Else If (Output[i]=')') Then dec(OpenTokens);
            End;
          Delete(Output, startC, 1);
          Insert('neg(', Output, startC);
          inc(i, 4);
          Insert(')', Output, i);
          i := startC+3;
        End;
      inc(i);
    End;

  //nutzlose Klammern entfernen
  i := 1;
  OpenTokens := 0;
  While i<=Length(Output) Do
    Begin
      If (Output[i]='(') Then
        Begin
          inc(OpenTokens);
          If OpenTokens=1 Then
            Begin
              startC := i;
              useful := false;
            End;
          While OpenTokens>1 Do
            Begin
              inc(i);
              If (Output[i]='(') Then inc(OpenTokens)
              Else If (Output[i]=')') Then dec(OpenTokens);
            End;
          inc(i);
        End;
      If OpenTokens=1 Then
        If IsOperator(Output[i])>Length(Operators) Then useful := true
      Else If Output[i]=')' Then
             Begin
               If (Useful=false) And
                  ((startC>1) And Not (Output[startC-1] In ['a'..'z'])) Then
                 Begin
                   Delete(Output, startC, 1);
                   Delete(Output, i-1, 1);
                 End;
               i := startC;
               OpenTokens := 0;
             End;
      inc(i);
    End;

  //Zahlen durch Variablen ersetzen
  i := 1;
  j := Length(Variables);
  //Anzahl Variablen
  While i<=Length(Output) Do
    Begin
      If (Output[i] In ['0'..'9']) Or
         ((i=1) And (Output[i] In ['+','-']) And
         (i+1<=Length(Output)) And (Output[i+1] In ['0'..'9'])) Or
         ((i>1) And (Output[i] In ['+','-']) And (Output[i-1] In ['*','^','/','(']) And
         (i+1<=Length(Output)) And (Output[i+1] In ['0'..'9'])) Then
        Begin
          RPart := 0;
          IPart := 0;
          RPartSet := false;
          IPartSet := false;
          startC := i;

          If Output[i] In ['+', '-'] Then inc(i);
          While (Output[i] In ['0'..'9', '.']) Do
            Begin
              inc(i);
              If (Output[i]='e') Or (Output[i]='E') Then
                Begin
                  inc(i);
                  If Output[i] In ['+', '-'] Then inc(i);
                End;
            End;
          If Output[i]='i' Then
            Begin
              IPart := StrToFloat(Copy(Output,startC,i-startC));
              IPartSet := true;
              inc(i);
            End
          Else
            Begin
              RPart := StrToFloat(Copy(Output,startC,i-startC));
              RPartSet := true;
            End;

          //existiert ein zweiter Part der Zahl?
          If Not ((startC>1) And (Output[startC-1] In ['^', '*', '/'])) And
             (Output[i] In ['+', '-']) And (Output[i+1] In ['0'..'9']) Then
            Begin
              start2C := i;
              If Output[i] In ['+', '-'] Then inc(i);
              While (Output[i] In ['0'..'9', '.']) Do
                Begin
                  inc(i);
                  If (Output[i]='e') Or (Output[i]='E') Then
                    Begin
                      inc(i);
                      If Output[i] In ['+', '-'] Then inc(i);
                    End;
                End;
              If Output[i]='i' Then
                Begin
                  If IPartSet Then i := start2C
                  Else
                    Begin
                      IPart := StrToFloat(Copy(Output,start2C,i-start2C));
                      inc(i);
                    End;
                End
              Else If RPartSet Then i := start2C
              Else RPart := StrToFloat(Copy(Output,start2C,i-start2C));
            End;

          endC := i;

          AddV(MakeComplex(RPart, IPart), i);
          Delete(Output,startC,endC-startC);
          Insert(Variables[i], Output, startC);
          i := startC+Length(Variables[i]);
        End
      Else inc(i);
    End;

  Result := 0;
End;
Function TCParser.DoParsing(s: String): NodePointer;

{Diese Funktion nimmt einen String in s und generiert einen NodePointer, der
den Operator mit der höchsten Priorität ausführt. Funktionen vor Potenz vor
Punkt- vor Strichrechnung. Als Parameter enthält dieser NodePointer zwei weitere
NodePointer, die durch rekursiven Aufruf dieser Funktion erzeugt werden.}

Var 
  j: integer;
  nmbrFunction: integer;
  TokenOpen: integer;
  lowestOperator, poslowestOperator: integer;
Begin
  new(Result);

  //stellt fest ob am Anfang des Ausdrucks eine Funktion steht
  nmbrFunction := -1;
  //sucht nach erlaubten Funktionen
  For j := 0 To High(Functions) Do
    If Copy(s, 1, Length(Functions[j])) = Functions[j] Then
      Begin
        nmbrFunction := j;
        //Verändere falls Funktion vorkommt
        break;
      End;

  If nmbrFunction <> -1 Then
    Begin
      //finde heraus, ob der ganze Ausdruck in der Funktion steht
      //ergo zählt er die Klammern
      TokenOpen := 1;
      For j := length(Functions[nmbrFunction]) + 2 To Length(s) Do
        Begin
          If s[j] = OpeningToken Then inc(TokenOpen)
          Else If s[j] = ClosingToken Then dec(TokenOpen);
          If TokenOpen = 0 Then
            Begin
              If j = length(s) Then TokenOpen := -1;
              break;
            End;
        End;
      //Wenn bis zum letzten Zeichen eine Klammer offen bleibt, dann ist der gesamte
      //Ausdruck in der Funktion
      If TokenOpen = -1 Then
        Begin
          //also generiert er einen Parameter für diese Funktion
          Result^.F := FunctionsFunctions[nmbrFunction];
          Result^.Res := 0;
          Result^.nmbrF := nmbrFunction;
          Result^.nmbrO := -1;
          Result^.nmbrV := -1;
          Result^.Par1 := DoParsing(Copy(s, 2 + length(Functions[nmbrFunction]), length(s) - length(
                          Functions[nmbrFunction]) - 2));
          Result^.Par2 := Nil;
          Exit;
        End;
    End;

  //Er sucht den Operator mit der höchsten Priorität
  TokenOpen := 0;
  lowestOperator := -1;
  poslowestOperator := 0;
  For j := 1 To length(s) Do
    Begin
      If (s[j] = OpeningToken) Then inc(TokenOpen)
      Else If (s[j] = ClosingToken) Then dec(TokenOpen);
      If TokenOpen > 0 Then continue;
      //Klammern überspringen

      If IsOperator(s[j]) < lowestOperator Then
        Begin
          lowestOperator := IsOperator(s[j]);
          poslowestOperator := j;
        End;
    End;

  //Feste Ausdrücke, die Funktionen sind hier nur Dummys
  //der Wert ist in Result^.Res gespeichert
  If lowestOperator = -1 Then //Endnode erreicht
    Begin
      If s = 'Z' Then
        Begin
          Result^.F := @ReturnZ;
          Result^.nmbrV := -1;
          AddZ(Result);
        End
      Else If s = 'C' Then
             Begin
               Result^.F := @ReturnC;
               Result^.nmbrV := -1;
               AddC(Result);
             End
      Else
        Begin
          j := FindV(s)
               Result^.F := Nil;
          Result^.nmbrV := j;
          Result^.Res := VariablesValues[j];
        End;
      Result^.nmbrF := -1;
      Result^.nmbrO := -1;
      Result^.Par1 := Nil;
      Result^.Par2 := Nil;
      Exit;
    End;

  Result^.F := OperatorFunctions[lowestOperator];
  Result^.nmbrO := lowestOperator;
  Result^.nmbrF := -1;
  Result^.nmbrV := -1;
  Result^.Res := 0;

  //Müssen im ersten Parameter Klammern entfernt werden?
  TokenOpen := 0;
  For j := 1 To poslowestOperator - 2 Do
    If s[j] = OpeningToken Then inc(TokenOpen)
    Else If s[j] = ClosingToken Then dec(TokenOpen);
  //Ersten Parameter generieren
  If (s[poslowestOperator - 1] = ClosingToken) And
     (s[1] = OpeningToken) And (TokenOpen = 1) Then
    Result^.Par1 := DoParsing(Copy(s, 2, poslowestOperator - 3))
  Else Result^.Par1 := DoParsing(Copy(s, 1, poslowestOperator - 1));

  //Müssen im zweiten Parameter Klammern entfernt werden?
  TokenOpen := 0;
  For j := poslowestOperator + 1 To length(s) - 1 Do
    If s[j] = OpeningToken Then inc(TokenOpen)
    Else If s[j] = ClosingToken Then dec(TokenOpen);
  //Zweiter Parameter generieren
  If (s[poslowestOperator + 1] = OpeningToken) And
     (s[length(s)] = ClosingToken) And (TokenOpen = 1) Then
    Result^.Par2 := DoParsing(Copy(s, poslowestOperator + 2, length(s) - poslowestoperator - 2))
  Else Result^.Par2 := DoParsing(Copy(s, poslowestOperator + 1, length(s) - poslowestoperator));
End;



{###########################################################################
 |                 Mathematische Operationen am Baum                       |
 ###########################################################################}
Procedure TCParser.SimplifyNode(N: NodePointer);

Var 
  Changed: boolean;
  tmp1: NodePointer;
  i: integer;
  b1, b2: boolean;
  IsF: boolean;
Begin
  If (N=Nil) Or (N^.Par1=Nil) Then exit
  Else SimplifyNode(N^.Par1);
  If N^.Par2=Nil Then exit
  Else SimplifyNode(N^.Par2);

  Changed := true;
  While Changed Do
    Begin
      Changed := false;

      //beide Operanden sind Variablen
      If (N^.Par1^.nmbrV<>-1) And (N^.Par2^.nmbrV<>-1) Then
        Begin
          //Node wird durch das Ergebnis der Operation ersetzt
          N^.Res := N^.F(N^.Par1.Res, N^.Par2.Res);
          AddV(N^.Res, i);
          N^.nmbrV := i;
          N^.nmbrO := -1;
          N^.nmbrF := -1;
          Dispose(N^.Par1);
          Dispose(N^.Par2);
          N^.F := Nil;
          N^.Par1 := Nil;
          N^.Par2 := Nil;
          Exit;
          //zweiter Operand ist Variable
        End
      Else If (N^.Par1^.nmbrV<>-1) Then
             Begin
               Case N^.nmbrO Of 
                 //falls Kommutativgesetz gilt, wird vertauscht
                 0,2: SwitchNode(N^.Par1, N^.Par2);
                 // Addition, Multiplikation
                 1:
                    Begin
                      // Subtraktion
                      If N^.Par2.Res=0 Then
                        Begin
                          //a - 0 = a
                          Dispose(N^.Par2);
                          ReplaceNode(N, N^.Par1);
                          Changed := true;
                        End;
                    End;
                 3,4:
                      Begin
                        // Division, Potenzfunktion
                        If N^.Par2.Res=1 Then
                          Begin
                            Dispose(N^.Par2);
                            ReplaceNode(N, N^.Par1);
                            Changed := true;
                          End;
                      End;
               End;
             End;
      //erster Operand ist Variable
      If (N^.Par1^.nmbrV<>-1) Then
        Begin
          If ((N^.nmbrO=0) And (N^.Par1.Res=0)) Or    //0 + a = a
             ((N^.nmbrO=2) And (N^.Par1.Res=1)) Then  //1 * a = a
            Begin
              Dispose(N^.Par1);
              ReplaceNode(N, N^.Par1);
              Changed := true;
            End;
        End;
    End;
End;


Procedure TCParser.Simplify;
Begin
  SimplifyNode(root);
End;

Procedure TCParser.SimplifyNode(N: NodePointer);

Var 
  Changed: boolean;
  tmp1: NodePointer;
  i: integer;
  b1, b2: boolean;
  IsF: boolean;
Begin
  If (N=Nil) Or (N^.Par1=Nil) Then exit
  Else SimplifyNode(N^.Par1);
  If N^.Par2=Nil Then exit
  Else SimplifyNode(N^.Par2);

  Changed := true;
  While Changed Do
    Begin
      Changed := false;

      //beide Operanden sind Variablen
      If (N^.Par1^.nmbrV<>-1) And (N^.Par2^.nmbrV<>-1) Then
        Begin
          //Node wird durch das Ergebnis der Operation ersetzt
          N^.Res := N^.F(N^.Par1.Res, N^.Par2.Res);
          AddV(N^.Res, i);
          N^.nmbrV := i;
          N^.nmbrO := -1;
          N^.nmbrF := -1;
          Dispose(N^.Par1);
          Dispose(N^.Par2);
          N^.F := Nil;
          N^.Par1 := Nil;
          N^.Par2 := Nil;
          Exit;
          //zweiter Operand ist Variable
        End
      Else If (N^.Par1^.nmbrV<>-1) Then
             Begin
               Case N^.nmbrO Of 
                 //falls Kommutativgesetz gilt, wird vertauscht
                 0,2: SwitchNode(N^.Par1, N^.Par2);
                 // Addition, Multiplikation
                 1:
                    Begin
                      // Subtraktion
                      If N^.Par2.Res=0 Then
                        Begin
                          //a - 0 = a
                          Dispose(N^.Par2);
                          ReplaceNode(N, N^.Par1);
                          Changed := true;
                        End;
                    End;
                 3,4:
                      Begin
                        // Division, Potenzfunktion
                        If N^.Par2.Res=1 Then
                          Begin
                            Dispose(N^.Par2);
                            ReplaceNode(N, N^.Par1);
                            Changed := true;
                          End;
                      End;
               End;
             End;
      //erster Operand ist Variable
      If (N^.Par1^.nmbrV<>-1) Then
        Begin
          If ((N^.nmbrO=0) And (N^.Par1.Res=0)) Or    //0 + a = a
             ((N^.nmbrO=2) And (N^.Par1.Res=1)) Then  //1 * a = a
            Begin
              Dispose(N^.Par1);
              ReplaceNode(N, N^.Par1);
              Changed := true;
            End;
        End;
    End;

End;

Function TCParser.CopyTree: NodePointer;
Begin
  Result := CopyNode(root);
End;



Procedure TCParser.Differentiate;
Begin
  DifferentiateNode(root);
  Simplify;
  PrintTree;
End;

Procedure TCParser.DifferentiateNode(N: NodePointer);

Var 
  i: integer;
  tmp1, tmp2, tmp3, tmp4: NodePointer;
  equal: integer;
Begin
  If N=Nil Then exit;
  If (N^.nmbrV <> -1) Or (N^.F=@ReturnC) Then
    Begin
      N^.F := Nil;
      N^.Par1 := Nil;
      N^.Par2 := Nil;
      N^.Res := 0;
      AddV(N^.Res, N^.nmbrV);
      exit;
    End;

  Case N^.nmbrO Of 
    //Summenregel
    0,1:
         Begin
           DifferentiateNode(N^.Par1);
           DifferentiateNode(N^.Par2);
           exit;
         End;
    //Produktregel
    2:
       Begin
         N^.F := @PAdd;
         New(tmp1);
         tmp1^.F := @PMul;
         tmp1^.Par1 := CopyNode(N^.Par1);
         DifferentiateNode(tmp1^.Par1);
         tmp1^.Par2 := N^.Par2;
         New(tmp2);
         tmp2^.F := @PMul;
         tmp2^.Par1 := N^.Par1;
         tmp2^.Par2 := CopyNode(N^.Par2);
         DifferentiateNode(tmp2^.Par2);
         N^.Par1 := tmp1;
         N^.Par2 := tmp2;
         exit;
       End;
    //Quotientenregel
    3:
       Begin
         N^.F := @PDiv;
         New(tmp3);
         tmp3^.F := @PSub;
         New(tmp1);
         tmp1^.F := @PMul;
         tmp1^.Par1 := CopyNode(N^.Par1);
         DifferentiateNode(tmp1^.Par1);
         tmp1^.Par2 := N^.Par2;
         New(tmp2);
         tmp2^.F := @PMul;
         tmp2^.Par1 := N^.Par1;
         tmp2^.Par2 := CopyNode(N^.Par2);
         DifferentiateNode(tmp2^.Par2);
         tmp3^.Par1 := tmp1;
         tmp3^.Par2 := tmp2;
         New(tmp4);
         tmp4^.F := @PSqr;
         tmp4^.Par1 := CopyNode(N^.Par2);
         tmp4^.Par2 := Nil;
         N^.Par1 := tmp3;
         N^.Par2 := tmp4;
         exit;
       End;
    //Potenzregel
    //g(x)^h(x) = exp(h(x)*ln(g(x)))
    4:
       Begin
         N^.F := @PExp;
         New(tmp1);
         //ln(g(x))
         tmp1^.F := @PLn;
         tmp1^.Par1 := N^.Par1;
         tmp1^.Par2 := Nil;
         New(tmp2);
         tmp2^.F := @PMul;
         tmp2^.Par1 := N^.Par2;
         tmp2^.Par2 := tmp1;
         N^.Par1 := tmp2;
         N^.Par2 := Nil;
         DifferentiateNode(N);
         exit;
       End;
  End;
  Case N^.nmbrF Of 
    //d/dx sqr(x) = 2*x = dbl(x)
    2:   N^.F := @PDouble;
    //d/dx ln(g(x)) = g'(x) / g(x)
    3:
       Begin
         N^.F := @PDiv;
         New(tmp1);
         tmp1 := CopyNode(N^.Par1);
         DifferentiateNode(tmp1);
         New(tmp2);
         tmp2 := N^.Par1;
         N^.Par1 := tmp1;
         N^.Par2 := tmp2;
       End;
    //d/dx exp(g(x)) = g'(x) * exp(g(x))
    4:
       Begin
         N^.F := @PMul;
         New(tmp1);
         tmp1 := CopyNode(N^.Par1);
         DifferentiateNode(tmp1);
         New(tmp2);
         tmp2^.F := @PExp;
         tmp2^.Par1 := N^.Par1;
         tmp2^.Par2 := Nil;
         N^.Par1 := tmp1;
         N^.Par2 := tmp2;
       End;
    //neg(x), dbl(x), etc...
    Else DifferentiateNode(N^.Par1);
  End;
  If N^.F = @ReturnZ Then
    Begin
      N^.F := Nil;
      N^.Res := 1;
      AddV(N^.Res, N^.nmbrV);
    End;
End;

Function TCParser.GetOutput: string;
Begin
  Result := FunctionToString(root, 0);
End;

Function TCParser.FunctionToString(N: NodePointer; priority: integer): string;

Var 
  i: integer;
  equal: boolean;
Begin
  Result := '-leer-';
  If N^.nmbrO <> -1 Then
    Begin
      i := N^.nmbrO;
      If i<priority Then
        Result := '('+FunctionToString(N^.Par1,i)+Operators[i]+FunctionToString(N^.Par2,i)+')'
      Else Result := FunctionToString(N^.Par1,i)+Operators[i]+FunctionToString(N^.Par2,i);
      exit;
    End
  Else If N^.nmbrF <> -1 Then Result := Functions[i]+'('+FunctionToString(N^.Par1,0)+')'
  Else If N^.nmbrV <> -1 Then Result := CompToStr(VariablesValues[i])
  Else If N^.F = @ReturnZ Then Result := 'Z'
  Else If N^.F = @ReturnC Then Result := 'C';
End;



constructor TCParser.Create;
Begin
  inherited;
  OpeningToken := '(';
  ClosingToken := ')';
  //Klammern, die die Terme abschließen
  root := Nil;
  ActiveC := 0;
  ActiveZ := 0;
  OperatorFunctions[0] := @PAdd;
  OperatorFunctions[1] := @PSub;
  OperatorFunctions[2] := @PMul;
  OperatorFunctions[3] := @PDiv;
  OperatorFunctions[4] := @PPower;
  FunctionsFunctions[0] := @PAbs;
  FunctionsFunctions[1] := @PMod;
  FunctionsFunctions[2] := @PSqr;
  FunctionsFunctions[3] := @PLn;
  FunctionsFunctions[4] := @PExp;
  FunctionsFunctions[5] := @PNeg;
  FunctionsFunctions[6] := @PDouble;
End;





Function TCParser.Parse(s: String): integer;
Begin
  ClearTree;
  Input := s;
  Result := PrepareParsing;
  root := DoParsing(Output);
  UpdateZ(ActiveZ);
  UpdateC(ActiveC);
  PrintTree;
End;

Procedure TCParser.PrintTree;
Begin
  depth := 0;
  printNode(root);
End;

Procedure TCParser.PrintNode(N: NodePointer);

Var 
  i: integer;
  s: string;
Begin
  inc(depth);
  s := '';
  For i := 1 To depth Do
    s := s + '----';
  s := s + '>  ';
  If N^.nmbrO <> -1 Then s := s + Operators[N^.nmbrO]
End
Else If N^.nmbrV <> -1 Then s := s + Variables[N^.nmbrV] + ':  ' + CompToStr(VariablesValues[i])
End
Else If N^.nmbrF <> -1 Then s := s + Functions[N^.nmbrF]
End
Else If N^.F = @ReturnZ Then s := s + 'Z'
Else If N^.F = @ReturnC Then s := s + 'C';

FG.mem_parseDebug.Lines.Add(s);
If N^.Par1 <> Nil Then PrintNode(N^.Par1);
If N^.Par2 <> Nil Then PrintNode(N^.Par2);
dec(depth);
End;

Procedure TCParser.ClearTree;
Begin
  If root = Nil Then exit;
  If root^.Par1 <> Nil Then DisposeNode(root^.Par1);
  If root^.Par2 <> Nil Then DisposeNode(root^.Par2);
  root := Nil;
End;

Function TCParser.Evaluate: TComplex;
Begin
  EvaluateNode(root);
  Result := root^.Res;
End;

Procedure TCParser.EvaluateNode(Var N: NodePointer);
Begin
  If N^.Par1=Nil Then exit
  Else
    Begin
      EvaluateNode(N^.Par1);
      N^.res := N^.Par1^.res;
      If N^.Par2<>Nil Then
        Begin
          EvaluateNode(N^.Par2);
          N^.F(N^.res, N^.Par2^.res);
        End
      Else N^.F(N^.res, N^.res);
    End;
End;

Function IsOperator(p: char): integer;

Var 
  i: integer;
Begin
  Result := High(Operators) + 1;
  For i := 0 To High(Operators) Do
    If p = Operators[i] Then
      Begin
        Result := i;
        Exit;
      End;
End;

Function IsFunction(s: String; pos: integer): boolean;

Var 
  i: integer;
Begin
  Result := false;
  For i := 0 To High(Functions) Do
    Begin
      If Copy(Functions[i], 1, Length(Functions[i]) - 1) = Copy(s, pos, Length(Functions[i]) - 1)
        Then
        Begin
          Result := true;
          Exit;
        End;
    End;
End;

Procedure TCParser.ReturnZ(Var C1, C2: TComplex);
Begin
  C1 := ActiveZ;
End;

Procedure TCParser.ReturnC(Var C1, C2: TComplex);
Begin
  C1 := ActiveC;
End;

Procedure TCParser.PAdd(Var C1, C2: TComplex); {$ifdef UseAsm}
assembler; {$endif}
{$ifdef UseAsm}
assembler;
asm                            {ST(0)       ST(1)   }
fld qword ptr [C1]           {C1.real             }
fld qword ptr [C2]           {C2.real     C1.real }
faddp ST(1), ST(0)           {result.real         }
fstp qword ptr [C1]          {                    }
fld qword ptr [C1+8]         {C1.imag             }
fld qword ptr [C2+8]         {C2.imag     C1.imag }
faddp ST(1), ST(0)           {result.imag}
fstp qword ptr [C1+8]
End;
{$else}
Begin
  C1.real := C1.real+C2.real;
  C1.imag := C1.imag+C2.imag;
End;
{$endif}


Procedure TCParser.PSub(Var C1, C2: TComplex);
assembler;
asm                            {ST(0)       ST(1)   }
fld qword ptr [C1]           {C1.real             }
fld qword ptr [C2]           {C2.real     C1.real }
fsubp ST(1), ST(0)           {result.real         }
fstp qword ptr [C1]      {                    }
fld qword ptr [C1+8]         {C1.imag             }
fld qword ptr [C2+8]         {C2.imag     C1.imag }
fsubp ST(1), ST(0)           {result.imag}
fstp qword ptr [C1+8]
End;

Procedure TCParser.PMul(Var C1, C2: TComplex);
assembler;
asm                            {ST(0)      ST(1)     ST(2)    ST(3)  }
fld qword ptr [C2]      {C2.real                              }
fld qword ptr [C2+8]                  {C2.imag    C2.real                   }
fld qword ptr [C1]      {C1.real    C2.imag   C2.real         }
fld qword ptr [C1+8]    {C1.imag    C1.real   C2.imag         C2.real}

fld ST(3)               {C2.real    C1.imag    C1.real   C2.imag         C2.real}
fmul ST(0), ST(2)       {C2.r*C1.r  C1.imag    C1.real   C2.imag         C2.real}
fld ST(1)               {C1.imag    C2.r*C1.r  C1.imag   C1.real   C2.imag         C2.real}
fmul ST(0), ST(4)       {C1.i*C2.i  C2.r*C1.r  C1.imag   C1.real   C2.imag         C2.real}
fsubp ST(1), ST(0)      {result.r   C1.imag    C1.real   C2.imag         C2.real}
fstp qword ptr [C1] {C1.imag    C1.real   C2.imag         C2.real}

fmulp ST(3), ST(0)      {C1.real    C2.imag   C1.imag*C2.real}
fmulp ST(1), ST(0)      {C1.r*C2.i  C1.i*C2.r}
faddp St(1), ST(0)      {result.imag}
fstp qword ptr [C1+8]
End;

Procedure TCParser.PDiv(Var C1, C2: TComplex);
assembler;
asm                       {ST(0)      ST(1)      ST(2)     ST(3)     ST(4)}
fld qword ptr [C2]      {C2.real                              }
fld qword ptr [C2+8]    {C2.imag    C2.real                   }
fld qword ptr [C1]      {C1.real    C2.imag   C2.real         }
fld qword ptr [C1+8]    {C1.imag    C1.real   C2.imag         C2.real}

fld ST(3)               {C2.real    C1.imag    C1.real   C2.imag         C2.real}
fmul ST(0), ST(0)       {C2.real²   C1.imag    C1.real   C2.imag         C2.real}
fld ST(3)               {C2.imag    C2.real²   C1.imag    C1.real   C2.imag         C2.real}
fmul ST(0), ST(0)       {C2.imag²   C2.real²   C1.imag    C1.real   C2.imag         C2.real}
faddp ST(1), ST(0)      {C2.i²+C2.r²C1.imag    C1.real   C2.imag         C2.real}
//  fld Smallest
//  faddp ST(1), ST(0)
fxch ST(4)              {C2.real    C1.real   C1.imag   C2.imag         divisor}

fld ST(0)               {C2.real    C2.real    C1.real   C1.imag   C2.imag         divisor}
fmul ST(0), ST(2)       {C2.r*C1.r  C2.real    C1.real   C1.imag   C2.imag         divisor}
fld ST(3)
{C1.imag    C2.r*C1.r  C2.real    C1.real   C1.imag   C2.imag         divisor}
fmul ST(0), ST(5)
{C1.i*C2.i  C2.r*C1.r  C2.real    C1.real   C1.imag   C2.imag         divisor}
faddp ST(1), ST(0)      {a          C2.real    C1.real   C1.imag   C2.imag         divisor}
fdiv ST(0), ST(5)       {result.r   C2.real    C1.real   C1.imag   C2.imag         divisor}
fstp qword ptr [C1] {C2.real    C1.real   C1.imag   C2.imag         divisor}

fmulp ST(2), ST(0)      {C1.real    C2.r*C1.i  C2.imag   divisor}
fmulp ST(2), ST(0)      {C2.r*C1.i  C1.r*C2.i  divisor}
fsubrp ST(1), ST(0)     {b          divisor}
fdivrp ST(1), ST(0)     {result.i}
fstp qword ptr [C1+8]
End;

Procedure TCParser.PPower(Var C1, C2: TComplex);
assembler;
asm                 {ST(0)  ST(1)  ST(2)  ST(3)  ST(4) }
fldln2            {ln2                               }
fld qword ptr [C1]       {C1.r   ln2                        }
fmul ST(0),ST(0)  {C1.r^2 ln2                        }
fld C1.imag       {C1.i   C1.r^2 ln2                 }
fmul ST(0),ST(0)  {C1.i^2 C1.r^2 ln2                 }
//  fld  Smallest     {Small  C1.i^2 C1.r^2 ln2          }
//  fadd              {S+i1^2 C1.r^2 ln2                 }
fadd              {modC1  ln2                        }
fsqrt             {r      ln2                        }
fyl2x             {ln(r)                             }

fld qword ptr [C1+8]       {C1.i   ln(r)                      }
fld qword ptr [C1]       {C1.r   C1.i   ln(r)               }
fpatan            {argC1  ln(r)                      }

fld qword ptr [C2]       {C2.r    argC1  ln(r)              }
fmul ST(0), ST(2) {r2*lnr  argC1  ln(r)              }
fld qword ptr [C2+8]       {C2.i    r2*lnr ln(r)              }
fmul ST(0), ST(2) {i2*arg  r2*lnr argC1  ln(r)       }
fsub              {a       argC1  ln(r)              }

fld qword ptr [C2]       {C2.r    a      argC1  ln(r)       }
fmul ST(0), ST(2) {r2*arg  a      argC1  ln(r)       }
fld qword ptr [C2+8]       {C2.i    r2*arg a      argC1  ln(r)}
fmul ST(0), ST(4) {i2*lnr  r2*arg a      argC1  ln(r)}
faddp ST(1), ST(0)             {b       a      argC1  ln(r)       }

fsincos           {cos(b)  sin(b) a      arg1   ln(r)}
fnstsw  ax        {Speichert das Status-Word des Coprozessors}
sahf              {Bringt AX ins Flagregister des Prozessors}
jpo    @@a1       {Parity-Flag des Prozessors entspricht nun}
fstp   ST(0)      {Denormalisations-Flag des Coprozessors.}
fld1              {Falls gesetzt überschreib cos(b) und sin(a)}
fld1              {mit Einsen}
@@a1:
      fstp ST(4)        {sin(b)  a      argC1  cos(b)}
      fstp ST(4)        {a       argC1  cos(b) sin(b)}
 {Prüft ob -MaxExp < a < MaxExp}
      fld MaxExp
      push eax          {sichere eax}
      fcom ST(1)        {vergleicht MaxExp mit a}
      fstsw ax          {bringt Statusword des Coprozessors}
      sahf              {ins Flagregister des Prozessors}
      jnc @@b1          {wenn Carry-Flag gesetzt (MaxExp>a)}
      fst ST(1)         {wenn nicht überschreibe a mit MaxExp}
      jmp @@c1
      @@b1:
            fchs              {Aus MaxExp wird -MaxExp}
            fcom ST(1)        {analog zu oben}
            fstsw ax
            sahf
            jc @@c1
            fst ST(1)
            @@c1:
                  fstp ST(0)        {überschreibe MaxExp und fahre fort}
                  pop eax           {hole eax zurück}

                    {ST(0)  ST(1)  ST(2)  ST(3)  ST(4) }
                  fldl2e            {lg2(e) a      argC1  cos(b) sin(b)}
                  fmulp st(1),st(0) {a*lg2e argC1  cos(b) sin(b)       }
                  fst   st(1)       {x      x      cos(b) sin(b)       }
                  frndint           {rnd(x) x      cos(b) sin(b)       }
                  fsub  st(1),st(0) {rnd(x) frc(x) cos(b) sin(b)       }
                  fxch              {frc(x) rnd(x) cos(b) sin(b)       }
                  f2xm1             {2^st-1 rnd(x) cos(b) sin(b)       }
                  fld1              {1      2^st-1 rnd(x) cos(b) sin(b)}
                  faddp ST(1), ST(0)             {2^st   rnd(x) cos(b) sin(b)       }
                  fscale            {2^x    rnd(x) cos(b) sin(b)       }
                  fstp    st(1)     {exp(a) cos(b) sin(b)              }

                  fmul ST(1), ST(0) {exp(a) Res.r  sin(b)              }
                  fmulp ST(2),ST(0) {Res.r  Res.i                      }
                  fstp  qword ptr [C1]  {Res.i                             }
                  fstp  qword ptr [C1+8]
End;

Procedure TCParser.PSqr(Var C1, C2: TComplex);
{$ifdef UseAsm}
asm
fld qword ptr [C1]
fld qword ptr [C1+8]
fld ST(1)
fmul ST(0),ST(0)
fld ST(1)
fmul ST(0),ST(0)
fsubp ST(1), ST(0)
fxch ST(2)

fmulp ST(1), ST(0)
fadd  ST(0), ST(0)
fstp qword ptr [C1+8]
fstp qword ptr [C1]
End;
{$else}

Var 
  tmp: Double;
Begin
  tmp := C1.real*C1.real-C1.imag*C1.imag;
  C1.imag := 2*C1.real*C1.imag;
  C1.real := tmp;
End;
{$endif}

Procedure TCParser.PAbs(Var C1, C2: TComplex);
assembler;
asm
fld qword ptr [C1]
fmul ST(0),ST(0)
fld qword ptr [C1+8]
fmul ST(0),ST(0)
faddp ST(1), ST(0)
fsqrt
fstp qword ptr [C1]
End;

Procedure TCParser.PMod(Var C1, C2: TComplex);
Begin
  C1.real := C1.real*C1.real+C1.imag*C1.imag;
End;

Procedure TCParser.PNeg(Var C1, C2: TComplex);
Begin
  C1.real := -C1.real;
  C1.imag := -C1.imag;
End;

Procedure TCParser.PDouble(Var C1, C2: TComplex);
Begin
  C1.real := C1.real+C1.real;
  C1.imag := C1.imag+C1.imag;
End;

Procedure TCParser.PLn(Var C1, C2: TComplex);

Var 
  tmp: TComplex;
Begin
  tmp.real := ln(CAbs(C1));
  tmp.imag := CArg(C1);
  C1 := tmp;
End;

Procedure TCParser.PExp(Var C1, C2: TComplex);

Var 
  r: double;
Begin
  r := exp(C1.real);
  C1.real := r*cos(C1.imag);
  C1.imag := r*sin(C1.imag);
End;

End.
