unit UTCParser;

interface
{$I cfg_symbols.inc}

uses UTComplex, SysUtils, strUtils, UMisc, UErrorHandling;

type TCPointer = ^TComplex;
type TCFunction = procedure(var C1, C2, R: TComplex);
//Eine komplexe Rechenoperation als Funktion
type
  OperatorPointer = ^TCOperator;
  TCOperator = record
    F: TCFunction;
    Par1, Par2, R: TCPointer;
  end;

const Operators: array[0..4] of Char = ('+', '-', '*', '/', '^');
const Functions: array[0..6] of string = ('abs', 'mod', 'sqr', 'ln', 'exp', 'neg', 'dbl');
//Die Elemente, die im Parser-String auftauchen dürfen
const OpeningToken = '(';
const ClosingToken = ')';
//Klammern: Man kann auch { oder & nehmen :-)

type
  NodePointer = ^TCNode;  //Ein Zeiger auf einen Ast im Baum
  TCNode = packed record  //Der Ast selber
    F: TCFunction; //Der Zeiger auf die Rechenoperation, die der Ast durchführt
    Par1, Par2: NodePointer; //Zeiger auf die Äste, die die Argumente liefern
    nmbrV, nmbrO, nmbrF: integer; //Nummer der Variable, des Operators, der Funktion
    Res: TComplex; //Zwischenspeicher für das Ergebnis von F, hält den Variablenwert
  end;

type
  TCParser = class(TObject)
  private
    root: NodePointer;                //Der Ursprung des Baumes
    Output,Input: string;             //Output, Input-string

    OperatorFunctions: array[0..4] of TCFunction;
    //enthält die Funktionen für Addition, Subtraktion...
    FunctionsFunctions: array[0..6] of TCFunction;
    //Funktionen für abs(), mod(), sqr(), ln(), ...
    Variables: array of string;
    //Namen der Variablen V1, V2, V3, ...
    VariablesValues: array of TComplex;
    //Werte der Variablen V1=x+i*y, ...

    //++++++++++Variablenverwaltung++++++++++
    function AddV(C: TComplex): integer;
    //Variable einfügen
    function FindV(s: string): integer;
    //Variablenindex über Name kriegen

    //++++++++++Baumverwaltung++++++++++
    procedure SwitchNode(var N1, N2: NodePointer);   //N1 = N2; N2 = N1;
    procedure ReplaceNode(var N1, N2: NodePointer);  //N1 = N2; Dispose(N2);
    function CopyNode(N: NodePointer): NodePointer;  //Result^ = N1^
    procedure DisposeNode(N: NodePointer);           //Dispose(N^.Par1, N^.Par2)
    procedure SetNode(N: NodePointer; o,f,v: integer);
    function CompareNode(N1, N2: NodePointer): boolean;
    procedure ReplaceNodewV(N: NodePointer; C: TComplex);

    //++++++++++Parsen++++++++++
    procedure PrepareParsing;
    //bereitet den Input-string fürs Parsen vor
    function RemoveUselessTokens(var s: string; var sPos, ePos: integer): boolean;
    //entfernt Klammern wie ((Z+C)+(Z)) -> (Z+C)+Z
    procedure CorrectTokens(var s: string; var sPos, ePos: integer);
    //setzt dringend benötigte Klammern: Z - Z - Z -> Z - (Z + Z)
    function DoParsing(s: string): NodePointer;
    //parst den String s rekursiv und bildet einen Baum
    EvaluationList: array of TCOperator;
    HighEvaluationList: integer;
    ResultPointer: TCPointer;
    procedure BuildEvaluationList;
    function AddFunctionToEvaluationList(N: NodePointer): TCPointer;

    //++++++++++Mathematische Operationen am Baum++++++++++
    procedure EvaluateByList;
    procedure DifferentiateNode(N: NodePointer);
    //differenziert einen Ast
    procedure SimplifyNode(var N: NodePointer);
    //vereinfacht einen Ast
    function CountZ(N: NodePointer): integer;
    //zählt wie oft Z in dem Ast vorkommt

    //++++++++++Helfer-Funktionen++++++++++
    function GetOutput: string;
    //gibt den geparsten String aus
    function FunctionToString(N: NodePointer; priority: integer): string;
    //wandelt einen geparsten Ast in einen Formel-String um
    procedure PrintNode(N: NodePointer; depth: integer);
    //Einen Ast mit der Tiefe depth im Memo ausgeben (Debug-Output)

  public
    constructor Create;
    Z: TComplex;
    C: TComplex;
    function Parse(s: string): boolean;
    //weist den Parser an, den string zu parsen und einen Baum aufzubauen
    procedure Differentiate;
    //Differenziert über Z
    procedure Simplify;
    //Vereinfacht
    procedure PrintTree;
    //gibt den kompletten Baum in einem Memo aus
    procedure ClearTree;
    //löscht den Baum ! (gibt den Speicher frei)
    function Evaluate: TComplex;
    //Wertet den Baum aus
    function IterateZ: TComplex;
    //Wertet den Baum aus und iteriert dabei über Z
    property ParsedString: string read GetOutput;
    //gibt den geparsten String zurück
  end;


//++++++++++komplexe Rechenoperationen++++++++++
//analog zu den Funktionen in UTComplex
    procedure PAdd(var C1, C2, R: TComplex); register;
    procedure PSub(var C1, C2, R: TComplex); register;
    procedure PMul(var C1, C2, R: TComplex); register;
    procedure PDiv(var C1, C2, R: TComplex); register;
    procedure PPower(var C1, C2, R: TComplex); register;
    procedure PPowerR(var C1, C2, R: TComplex); register;
    procedure PAbs(var C1, C2, R: TComplex);
    procedure PMod(var C1, C2, R: TComplex);
    procedure PSqr(var C1, C2, R: TComplex); register;
    procedure PLn(var C1, C2, R: TComplex);
    procedure PExp(var C1, C2, R: TComplex);
    procedure PNeg(var C1, C2, R: TComplex);
    procedure PDouble(var C1, C2, R: TComplex); register;
    procedure ReturnZ(var C1, C2, R: TComplex); //nur der Einfachheit halber
    procedure ReturnC(var C1, C2, R: TComplex);

//++++++++++Hilfsfunktionen++++++++++
function IsOperator(p: char): integer;
function IsFunction(s: string; pos: integer): boolean;
function IsComplexNumber(s: string; Pos: integer): boolean;
function ReadComplexNumber(s: string; sPos: integer; var ePos: integer): TComplex;
function GetClosingToken(s: string; sPos: integer): integer;
function GetNextOperator(s: string; sPos: integer): integer;

implementation

{$ifdef DebugParser}uses UTFractalGenerator;{$endif}


{###########################################################################
 |                       Variablenverwaltung                               |
 ###########################################################################}
function TCParser.AddV(C: TComplex): integer;
var
   found: boolean;
begin
     Result:=0;
     found:=false;
     while Result<=High(VariablesValues) do
     begin
          if VariablesValues[Result]=C then
          begin
               found:=true;
               break;
          end;
          inc(Result);
     end;
     if not found then
     begin
          Result:=Length(VariablesValues);
          SetLength(VariablesValues, Result+1);
          SetLength(Variables, Result+1);
          VariablesValues[Result]:=C;
          Variables[Result]:='V'+IntToStr(Result);
     end;
end;
function TCParser.FindV(s: string): integer;
begin
     Result:=0;
     while Result<=High(Variables) do
          if Variables[Result]=s then break
          else inc(Result);
end;

{###########################################################################
 |                          Baumverwaltung                                 |
 ###########################################################################}
procedure TCParser.SwitchNode(var N1, N2: NodePointer);
var
   tmp: NodePointer;
begin
     tmp:=N1;
     N1:=N2;
     N2:=tmp;
end;
procedure TCParser.ReplaceNode(var N1, N2: NodePointer);
var
   tmp: NodePointer;
begin
     tmp:=N1;
     N1:=N2;
     if tmp<>nil then Dispose(tmp);
end;
function TCParser.CopyNode(N: NodePointer): NodePointer;
begin
  if N=nil then Result:=nil
  else begin
     new(Result);
     Result^.F:=N^.F;
     Result^.Res:=N^.Res;
     Result^.nmbrO:=N^.nmbrO;
     Result^.nmbrF:=N^.nmbrF;
     Result^.nmbrV:=N^.nmbrV;
     Result^.Par1:=CopyNode(N^.Par1);
     Result^.Par2:=CopyNode(N^.Par2);
  end;
end;
procedure TCParser.DisposeNode(N: NodePointer);
begin
  if N <> nil then
  begin
    if N^.Par1 <> nil then DisposeNode(N^.Par1);
    if N^.Par2 <> nil then DisposeNode(N^.Par2);
    Dispose(N);
  end;
end;
procedure TCParser.SetNode(N: NodePointer; o,f,v: integer);
begin
  N^.nmbrO:=o;
  N^.nmbrF:=f;
  N^.nmbrV:=v;
  if o<>-1 then N^.F:=OperatorFunctions[o]
  else if f<>-1 then begin
    N^.F := FunctionsFunctions[f];
    N^.Par2 := nil;
  end else begin
    N^.Par1 := nil;
    N^.Par2 := nil;
  end;
end;
function TCParser.CompareNode(N1, N2: NodePointer): boolean;
begin
     Result:=true;
     if ((N1=nil) or (N2=nil)) then
     begin
          Result:=false;
          exit;
     end;
     if N1^.F<>N2^.F then Result:=false;
     if N1^.nmbrO<>N2^.nmbrO then Result:=false;
     if N2^.nmbrF<>N2^.nmbrF then Result:=false;
     if N2^.nmbrV<>N2^.nmbrV then Result:=false;
     if Result then Result:=CompareNode(N1^.Par1, N2^.Par1);
     if Result then Result:=CompareNode(N1^.Par2, N2^.Par2);
end;
procedure TCParser.ReplaceNodewV(N: NodePointer; C: TComplex);
begin
     if N=nil then exit;
     if N^.Par1<>nil then DisposeNode(N^.Par1);
     if N^.Par2<>nil then DisposeNode(N^.Par2);
     N^.F:=nil;
     N^.Par1:=nil;
     N^.Par2:=nil;
     N^.Res:=C;
     N^.nmbrV:=AddV(N^.Res);
     N^.nmbrO:=-1;
     N^.nmbrF:=-1;
end;


{###########################################################################
 |                             Parsen                                      |
 ###########################################################################}
procedure TCParser.PrepareParsing;
var
  i, j: integer;
  OpenTokens: integer;
  startC, endC: integer;
begin
  Output := Input;

  //Leerzeichen, CF und LF entfernen
  DelSpace(Output);
  Output :=  DelChars(Output, chr(10));
  Output :=  DelChars(Output, chr(13));
  Output := DelChars(Output, ' ');

  if IsEmptyStr(Output, [')','(']) then RaiseError($F1,0,0);

  //alle Buchstaben kleinschreiben
  Output := AnsiLowerCase(Output);

  //nicht erlaubte Zeichen im Eingabestring finden
  for i := 1 to Length(Output) do
    if not (Output[i] in ['a'..'z', '0'..'9', '.', OpeningToken, ClosingToken]) and
      not (Output[i] in ['+','-','/','*','^']) then
      RaiseError($F2, i);

  //Bezeichner (C + Z) werden großgeschrieben
  i := 1;
  while i <= Length(Output) do
  begin
    if (Output[i] in ['z', 'c']) and
      ( (i = 1) or not (Output[i-1] in ['a'..'z']) ) and
      ( (i = Length(Output)) or not (Output[i+1] in ['a'..'z']) ) then
        if Output[i] = 'z' then Output[i]:='Z'
        else if Output[i] = 'c' then Output[i]:='C';
    inc(i);
  end;

  //Zahlen durch Variablen ersetzen
  i:=1;
  while i<=Length(Output) do
  begin
    if IsComplexNumber(Output, i) then
    begin
      startC := i;
      i := AddV( ReadComplexNumber(Output, startC, endC) );
      Delete(Output,startC,endC-startC);
      Insert(Variables[i], Output, startC);
      i:=startC+Length(Variables[i]);
    end else inc(i);
  end;

  //unäres Minus durch neg() ersetzen
  //unäres Plus fällt weg
  i:=1;
  while i <= Length(Output) do
  begin
    if (Output[i]='-') and
      ( (i = 1) or (Output[i-1] = '(') ) then
    begin
      j := i;
      i := GetNextOperator(Output, i);
      Delete(Output, j, 1);
      Insert('neg(', Output, j);
      Insert(')', Output, i+3);
      i := j + 4;
    end else if (Output[i]='+') and
      ( (i = 1) or (Output[i-1] = '(') ) then
      Delete(Output, i, 1)
    else inc(i);
  end;

  {Klammern, die ganzen Ausdruck umschließen, müssen entfernt werden!
    z.B. (((Z*Z+C))) -> Z*Z + C}
  while (Output[1] = OpeningToken) and
    (GetClosingToken(Output, 1)=Length(Output)) do
  begin
    Delete(Output, 1, 1);
    Delete(Output, Length(Output), 1);
  end;

  {sonstige nutzlose Klammern: z.B. ((Z+C))+(C) -> Z+C+C}
  i := 1;
  j := Length(Output);
  RemoveUselessTokens(Output, i, j);

  {wichtig! Ausdrücke wie Z - Z - Z - Z werden in Z - (Z - (Z - Z)) = 0
   geparst. Richtig ist Z - (Z + Z + Z)! Deshalb müssen vorher zusätzliche
   Klammern gesetzt und Vorzeichen umgewandelt werden. }
  i := 1;
  j := Length(Output);
  CorrectTokens(Output, i, j);
end;

function TCParser.RemoveUselessTokens(var s: string; var sPos, ePos: integer): boolean;
var
  i, j: integer;
begin
  Result := false;
  i:=sPos;
  while i<=ePos do
  begin
    if (s[i] = OpeningToken) and not ((i > 1) and (s[i-1] in ['a'..'z'])) then
    begin
      j := GetClosingToken(s, i);
      inc(i);
      dec(j);
      if not RemoveUselessTokens(s, i, j) then
      begin
        Delete(s, i-1, 1);
        Delete(s, j, 1);
        dec(ePos, 2);
      end else i := j + 1;
    end;
    if IsOperator(s[i]) in [0..3] then Result:=true;
    inc(i);
  end;
end;
procedure TCParser.CorrectTokens(var s: string; var sPos, ePos: integer);
var
  i,j: integer;
  OldOperator, NewOperator: integer;
  OldOperatorPlace: integer;
  ReplacementMode: boolean;
begin
  ReplacementMode := false;
  OldOperator := -1;
  OldOperatorPlace := 1;
  i := sPos;
  while i <= ePos do
  begin
    if (s[i] = OpeningToken) then
    begin
      j := i + 1;
      i := GetClosingToken(s, i) - 1;
      CorrectTokens(s, j, i);
      inc(i, 2);
      NewOperator := 5;
    end else NewOperator := IsOperator(s[i]);

    if (NewOperator = OldOperator) and ((NewOperator=1) or (NewOperator=3)) then
    begin
      ReplacementMode := true;
      case NewOperator of
        1:  s[i] := '+';
        3:  s[i] := '*';
      end;
    end else if (NewOperator < 4) or (i = Length(s)) then
    begin
      if ReplacementMode=true then
      begin
        if i=Length(s) then inc(i);
        ReplacementMode := false;
        Insert(OpeningToken, s, OldOperatorPlace+1);
        Insert(ClosingToken, s, i+1);
        inc(i,2);
        inc(ePos, 2);
      end;
      OldOperator := NewOperator;
      OldOperatorPlace := i;
    end;
    inc(i);
  end;
end;

function TCParser.DoParsing(s: string): NodePointer;
{Diese Funktion nimmt einen String in s und generiert einen NodePointer, der
den Operator mit der höchsten Priorität ausführt. Funktionen vor Potenz vor
Punkt- vor Strichrechnung. Als Parameter enthält dieser NodePointer zwei weitere
NodePointer, die durch rekursiven Aufruf dieser Funktion erzeugt werden.}
var
  j, i: integer;
  nmbrFunction: integer;
  TokenOpen: integer;
  lowestOperator, poslowestOperator: integer;
begin
  new(Result);

//stellt fest ob am Anfang des Ausdrucks eine Funktion steht
  nmbrFunction := -1;
  //sucht nach erlaubten Funktionen
  for j := 0 to High(Functions) do
    if Copy(s, 1, Length(Functions[j])) = Functions[j] then
    begin
      nmbrFunction := j; //Verändere falls Funktion vorkommt
      break;
    end;

  if nmbrFunction <> -1 then
  begin
//finde heraus, ob der ganze Ausdruck in der Funktion steht
    j := length(Functions[nmbrFunction]) + 1; //s[j] = OpeningToken
    i := GetClosingToken(s, j);
//Wenn die schließende Klammer das letzte Zeichen ist, dann ist der gesamte
//Ausdruck in der Funktion
    if i = length(s) then
    begin
//also generiert er einen Parameter für diese Funktion
      Result^.F := FunctionsFunctions[nmbrFunction];
      Result^.Res := 0;
      Result^.nmbrF := nmbrFunction;
      Result^.nmbrO := -1;
      Result^.nmbrV := -1;
      Result^.Par1 := DoParsing(Copy(s, j+1, i - j - 1));
      Result^.Par2 := nil;
      Exit;
    end;
  end;

//Er sucht den Operator mit der niedrigsten Priorität
  lowestOperator := High(Operators)+1;
  poslowestOperator := 0;
  j := 1;
  while j < length(s) do
  begin
    j := GetNextOperator(s, j);
    if (j<Length(s)) and (IsOperator(s[j]) < lowestOperator) then
    begin
      lowestOperator := IsOperator(s[j]);
      poslowestOperator := j;
    end;
  end;

  if lowestOperator = High(Operators)+1 then //Endnode erreicht
  //s muss Variable sein
  begin
    SetNode(Result, -1, -1, -1);
    if s = 'Z' then Result^.F := @ReturnZ
    else if s = 'C' then Result^.F := @ReturnC
    else begin
      j := FindV(s);
      if j > High(VariablesValues) then RaiseError($F3);
      SetNode(Result, -1, -1, j);
      Result^.Res := VariablesValues[j];
    end;
    Exit;
  end;

  SetNode(Result, lowestOperator, -1, -1);

  {//Müssen im ersten Parameter Klammern entfernt werden?
  TokenOpen := 0;
  for j := 1 to poslowestOperator - 2 do
    if s[j] = OpeningToken then inc(TokenOpen)
    else if s[j] = ClosingToken then dec(TokenOpen);
  //Ersten Parameter generieren
  if (s[poslowestOperator - 1] = ClosingToken) and
    (s[1] = OpeningToken) and (TokenOpen = 1) then}
  if (s[1] = OpeningToken) and (GetClosingToken(s,1)=poslowestOperator-1) then
    Result^.Par1 := DoParsing(Copy(s, 2, poslowestOperator - 3))
  else Result^.Par1 := DoParsing(Copy(s, 1, poslowestOperator - 1));

  {//Müssen im zweiten Parameter Klammern entfernt werden?
  TokenOpen := 0;
  for j := poslowestOperator + 1 to length(s) - 1 do
    if s[j] = OpeningToken then inc(TokenOpen)
    else if s[j] = ClosingToken then dec(TokenOpen);
  //Zweiter Parameter generieren
  if (s[poslowestOperator + 1] = OpeningToken) and
    (s[length(s)] = ClosingToken) and (TokenOpen = 1) then}
  if (s[poslowestOperator+1] = OpeningToken) and
    (GetClosingToken(s,poslowestOperator+1)=length(s)) then
    Result^.Par2 := DoParsing(Copy(s, poslowestOperator + 2, length(s) - poslowestoperator - 2))
  else Result^.Par2 := DoParsing(Copy(s, poslowestOperator + 1, length(s) - poslowestoperator));
end;

procedure TCParser.BuildEvaluationList;
begin
  HighEvaluationList := -1;
  SetLength(EvaluationList, 0);
  {$ifdef DebugParser}FG.mem_parseDebug.Lines.Add('List:');{$endif}
  AddFunctionToEvaluationList(root);
  {$ifdef DebugParser}FG.mem_parseDebug.Lines.Add('Listend.');{$endif}
  if root^.F = @ReturnZ then ResultPointer := @Z
  else if root^.F = @ReturnC then ResultPointer := @C
  else ResultPointer := @root^.Res;
end;

function TCParser.AddFunctionToEvaluationList(N: NodePointer): TCPointer;
var
  COp: TCOperator;
begin
  if N=nil then
  begin
    Result := nil;
    exit;
  end else begin
    if (N^.F = @ReturnC) then Result := @C
    else if (N^.F = @ReturnZ) then Result := @Z
    else Result := @N^.Res;
  end;
  COp.Par1 := AddFunctionToEvaluationList(N^.Par1);
  COp.Par2 := AddFunctionToEvaluationList(N^.Par2);
  if (COp.Par1=nil) and (COp.Par2 = nil) then exit;
  COp.F := N^.F;
  COp.R := @N^.Res;
  inc(HighEvaluationList);
  SetLength(EvaluationList, HighEvaluationList+1);
  {$ifdef DebugParser}
  with FG.mem_parseDebug.Lines do
  begin
    Add(IntToStr(HighEvaluationList)+' Op:'+IntToStr(N^.nmbrO)+' V:'+IntToStr(N^.nmbrV));
  end;
  {$endif}

  EvaluationList[HighEvaluationList] := COp;
end;

{###########################################################################
 |                 Mathematische Operationen am Baum                       |
 ###########################################################################}
procedure TCParser.EvaluateByList;
{$ifdef UseAsm}
assembler;
asm
  push ebx
  mov ebx, TCParser([eax]).EvaluationList
  mov ecx, TCParser([eax]).HighEvaluationList
  inc ecx
  jecxz @@endloop
@@beginloop:
  push ecx
  mov eax, TCOperator([ebx]).Par1
  mov edx, TCOperator([ebx]).Par2
  mov ecx, TCOperator([ebx]).R
  call TCOperator([ebx]).F
  pop ecx
  add ebx, 16
  loop @@beginloop
@@endloop:
  pop ebx
end;
{$else}
inline;
var
  i: integer;
  COp: ^TCOperator;
begin
  COp := @EvaluationList[0];
  for i:=0 to HighEvaluationList do
  begin
    COp^.F( COp^.Par1^, COp^.Par2^, COp^.R^);
    inc(COp);
  end;
end;
{$endif}

procedure TCParser.SimplifyNode(var N: NodePointer);
var
   Changed: boolean;
   tmp, tmp2: TComplex;
   Ntmp: NodePointer;
begin
if (N=nil) or (N^.Par1=nil) then exit else SimplifyNode(N^.Par1);
if N^.Par2<>nil then SimplifyNode(N^.Par2);

Changed:=true;
while Changed do
begin
     Changed:=false;

     //exp(a*ln(b)) = b ^ a
     if (N^.nmbrF=4) and (N^.Par1 <> nil) and (N^.Par1^.nmbrO=2) then
     begin
        Ntmp:=N^.Par1;
        if (Ntmp^.Par1 <> nil) and (Ntmp^.Par1^.nmbrF=3) then SwitchNode(Ntmp^.Par1, Ntmp^.Par2);
        if (Ntmp^.Par2 <> nil) and (Ntmp^.Par2^.nmbrF=3) then
        begin
             SetNode(N, 4, -1, -1);
             N^.Par1:=Ntmp^.Par2^.Par1;
             N^.Par2:=Ntmp^.Par1;
             Dispose(Ntmp^.Par2);
             Dispose(Ntmp);
             Changed:=True;
             Continue;
        end;
     end;

     //Funktion einer Konstanten
     if (N^.nmbrF<>-1) and (N^.Par1^.nmbrV<>-1) then
     begin
          tmp:=N^.Par1^.Res;
          N^.F(tmp, N^.Res, tmp2);
          ReplaceNodewV(N, tmp2);
          Exit;
     end;

     if N^.Par2=nil then exit;

     //Erster und zweiter Parameter sind _exakt_ gleich
     //bzw. die angehängten Äste sind gleich aufgebaut
     if CompareNode(N^.Par1, N^.Par2) then
     begin
          case N^.nmbrO of
               0:   begin
                         DisposeNode(N^.Par2);
                         SetNode(N, -1, 6, -1); //a + a = dbl(a)
                         Changed:=true;
                         Continue;
                    end;
               1:   ReplaceNodewV(N, 0);        //a - a = 0
               2:   begin
                         DisposeNode(N^.Par2);
                         SetNode(N, -1, 2, -1); //a * a = sqr(a)
                         Changed:=true;
                         Continue;
                    end;
               3:   ReplaceNodewV(N, 1);        //a / a = 1
          end;
          exit;
     end;

     //beide Operanden sind Variablen
     if (N^.Par1^.nmbrV<>-1) and (N^.Par2^.nmbrV<>-1) then
     begin
          //Fehler abfangen, z.B. 0^x <> 0, sondern 1E-15...
          if (N^.Par1^.Res = 0) and
            (N^.nmbrO in [2,3,4]) then ReplaceNodewV(N, 0)
          else begin
            //Node wird durch das Ergebnis der Operation ersetzt
            N^.F(N^.Par1^.Res, N^.Par2^.Res, tmp);
            ReplaceNodewV(N, tmp);
          end;
          Exit;
     //zweiter Operand ist Konstante
     end else if (N^.Par2^.nmbrV<>-1) then
     begin
          case N^.nmbrO of
               //falls Kommutativgesetz gilt, wird vertauscht
               0,2: SwitchNode(N^.Par1, N^.Par2);   // Addition, Multiplikation
               1:   begin              // Subtraktion
                         if N^.Par2^.Res=0 then
                         begin
                              //a - 0 = a
                              Dispose(N^.Par2);
                              ReplaceNode(N, N^.Par1);
                              Exit;
                         end;
                    end;
               3,4: begin    // Division, Potenzfunktion
                         if N^.Par2^.Res=1 then
                         begin
                              Dispose(N^.Par2);
                              ReplaceNode(N, N^.Par1);
                              Exit;
                         end;
                         if (N^.nmbrO=4) and (N^.Par2^.Res.imag=0) then
                         begin
                              N^.F:=@PPowerR;
                              Exit;
                         end;
                         //ersetzt C^Z durch C^R
                    end;
          end;
     end;
     //erster Operand ist Variable
     if (N^.Par1^.nmbrV<>-1) then
     begin
          if ((N^.nmbrO=0) and (N^.Par1^.Res=0)) or    //0 + a = a
             ((N^.nmbrO=2) and (N^.Par1^.Res=1)) then  //1 * a = a
          begin
               Dispose(N^.Par1);
               ReplaceNode(N, N^.Par2);
               Exit;
          end else if ((N^.nmbrO=2) and (N^.Par1^.Res=0)) or   //0 * a = 0
                      ((N^.nmbrO=3) and (N^.Par1^.Res=0)) then //0 / a = 0
               ReplaceNodewV(N, 0);
     end;
end;
end;
procedure TCParser.DifferentiateNode(N: NodePointer);
//Differenziert über Z
var
   tmp1, tmp2, tmp3, tmp4: NodePointer;
begin
     if N=nil then exit;

     if (N^.nmbrV <> -1) or (N^.F=@ReturnC) then
     begin
          N^.F := nil;
          N^.Par1:=nil;
          N^.Par2:=nil;
          N^.Res:=0;
          N^.nmbrV:=AddV(N^.Res);
          N^.nmbrF:=-1;
          N^.nmbrO:=-1;
          exit;
     end;

     if N^.F = @ReturnZ then
     begin
          N^.F := nil;
          N^.Res := 1;
          N^.nmbrV:=AddV(N^.Res);
          N^.nmbrO:=-1;
          N^.nmbrF:=-1;
          exit;
     end;

     case N^.nmbrO of
          //Summenregel
          0,1: begin
                    DifferentiateNode(N^.Par1);
                    DifferentiateNode(N^.Par2);
                    exit;
               end;
          //Produktregel d/dx h(x)*g(x) = h'(x)*g(x) + g'(x)*h(x)
          2:   begin
                    SetNode(N, 0, -1, -1);

                    New(tmp1);     //h'(x)*g(x)
                    SetNode(tmp1, 2, -1, -1);
                    tmp1^.Par1:=CopyNode(N^.Par1);
                    DifferentiateNode(tmp1^.Par1);
                    tmp1^.Par2:=N^.Par2;

                    New(tmp2);    //g'(x)*h(x)
                    SetNode(tmp2, 2, -1, -1);
                    tmp2^.Par1:=N^.Par1;
                    tmp2^.Par2:=CopyNode(N^.Par2);
                    DifferentiateNode(tmp2^.Par2);

                    N^.Par1:=tmp1;
                    N^.Par2:=tmp2;
                    exit;
               end;
          //Quotientenregel       //d/dx h(x)/g(x) = (h'(x)*g(x) - g'(x)*h(x)) / g(x)^2
          3:   begin
                    SetNode(N, 3, -1, -1);

                    New(tmp1);
                    SetNode(tmp1, 2, -1, -1);
                    tmp1^.Par1:=CopyNode(N^.Par1);
                    DifferentiateNode(tmp1^.Par1);
                    tmp1^.Par2:=N^.Par2;

                    New(tmp2);
                    SetNode(tmp2, 2, -1, -1);
                    tmp2^.Par1:=N^.Par1;
                    tmp2^.Par2:=CopyNode(N^.Par2);
                    DifferentiateNode(tmp2^.Par2);

                    New(tmp3);
                    SetNode(tmp3, 1, -1, -1);
                    tmp3^.Par1:=tmp1;
                    tmp3^.Par2:=tmp2;

                    New(tmp4);
                    SetNode(tmp4, -1, 2, -1);
                    tmp4^.Par1:=CopyNode(N^.Par2);
                    tmp4^.Par2:=nil;

                    N^.Par1:=tmp3;
                    N^.Par2:=tmp4;
                    exit;
               end;
               //Potenzregel
               //g(x)^h(x) = exp(h(x)*ln(g(x)))
          4:   if (N^.Par2^.nmbrV<>-1)
                  and (N^.Par1^.F = @ReturnZ) then
               begin
                    SetNode(N, 2, -1, -1);

                    New(tmp1);
                    SetNode(tmp1, 4, -1, -1);
                    tmp1^.Par1:=N^.Par1;
                    New(tmp1^.Par2);
                    tmp1^.Par2^.Par1:=nil;
                    tmp1^.Par2^.Par2:=nil;
                    ReplaceNodewV(tmp1^.Par2, VariablesValues[N^.Par2^.nmbrV] - 1);
                    N^.Par1:=tmp1;
                    exit;
               end else begin
                    New(tmp1);                  //ln(g(x))
                    SetNode(tmp1, -1, 3, -1);
                    tmp1^.Par1:=N^.Par1;
                    tmp1^.Par2:=nil;

                    New(tmp2);
                    SetNode(tmp2, 2, -1, -1);
                    tmp2^.Par1:=N^.Par2;
                    tmp2^.Par2:=tmp1;

                    SetNode(N, -1, 4, -1);
                    N^.Par1:=tmp2;
                    N^.Par2:=nil;
                    DifferentiateNode(N);
                    exit;
               end;
     end;

     //jeweils mit Kettenregel
     case N^.nmbrF of
          //d/dx sqr(g(x)) = dbl(x) * g'(x)
          2:   begin
                    New(tmp1);
                    tmp1 := CopyNode(N);
                    SetNode(tmp1, -1, 6, -1);
                    New(tmp2);
                    tmp2 := CopyNode(N^.Par1);
                    SetNode(N, 2, -1, -1);
                    N^.Par1 := tmp1;
                    N^.Par2 := tmp2;
                    DifferentiateNode(N^.Par2);
               end;
          //d/dx ln(g(x)) = g'(x) / g(x)
          3:   begin
                    SetNode(N, 3, -1, -1);
                    New(tmp1);
                    tmp1:=CopyNode(N^.Par1);
                    DifferentiateNode(tmp1);
                    New(tmp2);
                    tmp2:=N^.Par1;
                    N^.Par1:=tmp1;
                    N^.Par2:=tmp2;
               end;
          //d/dx exp(g(x)) = g'(x) * exp(g(x))
          4:   begin
                    SetNode(N, 2, -1, -1);

                    New(tmp1);
                    tmp1:=CopyNode(N^.Par1);
                    DifferentiateNode(tmp1);

                    New(tmp2);
                    SetNode(tmp2, -1, 4, -1);
                    tmp2^.Par1:=N^.Par1;
                    tmp2^.Par2:=nil;

                    N^.Par1:=tmp1;
                    N^.Par2:=tmp2;
               end;
           //neg(x), dbl(x), etc...
           else DifferentiateNode(N^.Par1);
     end;
end;

function TCParser.CountZ(N: NodePointer): integer;
begin
     if N^.F=@ReturnZ then Result:=1
     else Result:=0;
end;


{###########################################################################
 |                          Helferfunktionen                               |
 ###########################################################################}
function TCParser.GetOutput: string;
begin
  Result := '';
  if root <> nil then Result:=FunctionToString(root, 0);
end;
function TCParser.FunctionToString(N: NodePointer; priority: integer): string;
begin
     Result:=''; //'-leer-';
     if N^.nmbrO <> -1 then
     begin
          if N^.nmbrO<priority then Result := '('
          else Result := '';
          Result := Result + FunctionToString(N^.Par1, N^.nmbrO);
          Result := Result + Operators[N^.nmbrO];
          Result := Result + FunctionToString(N^.Par2, N^.nmbrO);
          if N^.nmbrO<priority then Result := Result + ')';
     end else if N^.nmbrF <> -1 then
         Result := Functions[N^.nmbrF]+'('+FunctionToString(N^.Par1,0)+')'
     else if N^.nmbrV <> -1 then Result := CompToStr(VariablesValues[N^.nmbrV],15)
     else if N^.F = @ReturnZ then Result := 'Z'
     else if N^.F = @ReturnC then Result := 'C';
end;
procedure TCParser.PrintNode(N: NodePointer; depth: integer);
var
  i: integer;
  base, s: string;
begin
  s := '';
  for i := 1 to depth do s := s + '----';
  s := s + '>  ';
  if N^.nmbrO <> -1 then s := s + Operators[N^.nmbrO]
  else if N^.nmbrV <> -1 then s := s + Variables[N^.nmbrV] + ':  ' +
       CompToStr(VariablesValues[N^.nmbrV])
  else if N^.nmbrF <> -1 then s := s + Functions[N^.nmbrF]
  else if N^.F = @ReturnZ then s := s + 'Z'
  else if N^.F = @ReturnC then s := s + 'C';


  {$ifdef DebugParser}
  FG.mem_parseDebug.Lines.Add(s);
  base := '';
  for i := 0 to depth do base := base + '    ';
  s:=base+'nmbrO: '+IntToStr(N^.nmbrO);
  FG.mem_parseDebug.Lines.Add(s);
  s:=base+'nmbrF: '+IntToStr(N^.nmbrF);
  FG.mem_parseDebug.Lines.Add(s);
  s:=base+'nmbrV: '+IntToStr(N^.nmbrV);
  FG.mem_parseDebug.Lines.Add(s);

  s:=base+'Res:   '+CompToStr(N^.Res);
  FG.mem_parseDebug.Lines.Add(s);

  FG.mem_parseDebug.Lines.Add(base+'Par1:');
  if N^.Par1 <> nil then PrintNode(N^.Par1, depth+1)
  else FG.mem_parseDebug.Lines.Add(base+'nil');
  FG.mem_parseDebug.Lines.Add(base+'Par2:');
  if N^.Par2 <> nil then PrintNode(N^.Par2, depth+1)
  else FG.mem_parseDebug.Lines.Add(base+'nil');
  {$endif}

  dec(depth);
end;


{###########################################################################
 |                          Public                                         |
 ###########################################################################}
constructor TCParser.Create;
begin
  inherited;
  root := nil;
  C:=0;
  Z:=0;
  //Funktionstabelle aufbauen
  OperatorFunctions[0] := @PAdd;
  OperatorFunctions[1] := @PSub;
  OperatorFunctions[2] := @PMul;
  OperatorFunctions[3] := @PDiv;
  OperatorFunctions[4] := @PPower;
  FunctionsFunctions[0] := @PAbs;
  FunctionsFunctions[1] := @PMod;
  FunctionsFunctions[2] := @PSqr;
  FunctionsFunctions[3] := @PLn;
  FunctionsFunctions[4] := @PExp;
  FunctionsFunctions[5] := @PNeg;
  FunctionsFunctions[6] := @PDouble;
end;

function TCParser.Parse(s: string): boolean;
begin
try
  Result := true;
  ClearTree;
  Input := s;
  PrepareParsing;
  root := DoParsing(Output);
  Simplify;
  BuildEvaluationList;
except
  Result := false;
end;
end;

procedure TCParser.Differentiate;
begin
     DifferentiateNode(root);
     Simplify;
     BuildEvaluationList;
end;

procedure TCParser.Simplify;
begin
     SimplifyNode(root);
     BuildEvaluationList;
end;

procedure TCParser.PrintTree;
begin
  printNode(root, 1);
end;

procedure TCParser.ClearTree;
begin
  SetLength(Variables, 0);
  SetLength(VariablesValues, 0);
  DisposeNode(root);
  root := nil;
end;

function TCParser.Evaluate: TComplex; inline;
begin
  EvaluateByList;
  Result := ResultPointer^;
end;

function TCParser.IterateZ: TComplex; inline;
begin
  EvaluateByList;
  Result := ResultPointer^;
  Z := Result;
end;


{###########################################################################
 |                        Komplexe Operationen                             |
 ###########################################################################}
procedure PAdd(var C1, C2, R: TComplex);
{$ifdef UseAsm}
assembler;
{$ifdef UseSSE}
asm
  movupd xmm0, [C1]
  movupd xmm1, [C2]
  addpd xmm0, xmm1
  movupd [R], xmm0
end;
{$else}
asm                            {ST(0)       ST(1)   }
  fld qword ptr [C1]           {C1.real             }
  fld qword ptr [C2]           {C2.real     C1.real }
  faddp ST(1), ST(0)           {result.real         }
  fstp qword ptr [R]          {                    }
  fld qword ptr [C1+8]         {C1.imag             }
  fld qword ptr [C2+8]         {C2.imag     C1.imag }
  faddp ST(1), ST(0)           {result.imag}
  fstp qword ptr [R+8]
end;
{$endif}
{$else}
register;
begin
  R.real:=C1.real+C2.real;
  R.imag:=C1.imag+C2.imag;
end;
{$endif}

procedure PSub(var C1, C2, R: TComplex);
{$ifdef UseAsm}
assembler;
asm                            {ST(0)       ST(1)   }
  fld qword ptr [C1]           {C1.real             }
  fld qword ptr [C2]           {C2.real     C1.real }
  fsubp ST(1), ST(0)           {result.real         }
  fstp qword ptr [R]           {                    }
  fld qword ptr [C1+8]         {C1.imag             }
  fld qword ptr [C2+8]         {C2.imag     C1.imag }
  fsubp ST(1), ST(0)           {result.imag}
  fstp qword ptr [R+8]
end;
{$else}
register;
begin
  R.real:=C1.real-C2.real;
  R.imag:=C1.imag-C2.imag;
end;
{$endif}

procedure PMul(var C1, C2, R: TComplex);
{$ifdef UseAsm}
assembler;
asm                            {ST(0)      ST(1)     ST(2)    ST(3)  }
  fld qword ptr [C2]      {C2.real                              }
  fld qword ptr [C2+8]                  {C2.imag    C2.real                   }
  fld qword ptr [C1]      {C1.real    C2.imag   C2.real         }
  fld qword ptr [C1+8]    {C1.imag    C1.real   C2.imag         C2.real}

  fld ST(3)               {C2.real    C1.imag    C1.real   C2.imag         C2.real}
  fmul ST(0), ST(2)       {C2.r*C1.r  C1.imag    C1.real   C2.imag         C2.real}
  fld ST(1)               {C1.imag    C2.r*C1.r  C1.imag   C1.real   C2.imag         C2.real}
  fmul ST(0), ST(4)       {C1.i*C2.i  C2.r*C1.r  C1.imag   C1.real   C2.imag         C2.real}
  fsubp ST(1), ST(0)      {result.r   C1.imag    C1.real   C2.imag         C2.real}
  fstp qword ptr [R] {C1.imag    C1.real   C2.imag         C2.real}

  fmulp ST(3), ST(0)      {C1.real    C2.imag   C1.imag*C2.real}
  fmulp ST(1), ST(0)      {C1.r*C2.i  C1.i*C2.r}
  faddp St(1), ST(0)      {result.imag}
  fstp qword ptr [R+8]
end;
{$else}
register;
begin
     R.real:=C1.real*C2.real-C1.imag*C2.imag;
     R.imag:=C1.real*C2.imag+C2.real*C1.imag;
end;
{$endif}

procedure PDiv(var C1, C2, R: TComplex);
{$ifdef UseAsm}
assembler;
{$ifdef UseSSE}
asm                   {xmm0         xmm1        xmm2        xmm3      xmm4      }
  movupd xmm0, [C1]   {b1|a1                                                    }
  movupd xmm1, [C2]   {b1|a1        b2|a2                                       }
  movapd xmm2, xmm0   {b1|a1        b2|a2       b1|a1                           }
  movapd xmm3, xmm1   {b1|a1        b2|a2       b1|a1       b2|a2              }

  unpckhpd xmm2, xmm1 {b1|a1        b2|a2       b2|b1       b2|a2 }
  unpcklpd xmm1, xmm0 {b1|a1        a1|a2       b2|b1       b2|a2 }
  mulpd xmm0, xmm3    {b1*b2|a1*a2  a1|a2       b2|b1       b2|a2 }
  mulpd xmm3, xmm3
  mulpd xmm1, xmm2    {b1*b2|a1*a2  a1*b2|a2*b1 b2|b1       b2|a2 }
  haddpd xmm3, xmm3
  hsubpd xmm1, xmm1   {b1*b2|a1*a2  y|y         b2|b1       b2|a2 }
  haddpd xmm0, xmm0   {x|x          y|y         b2|b1       b2|a2 }
  unpcklpd xmm0, xmm1 {y|x          y|y         b2|b1       b2|a2 }
  divpd xmm0, xmm3
  movupd [R], xmm0
end;
{$else}
asm                       {ST(0)      ST(1)      ST(2)     ST(3)     ST(4)      ST(5)     }
  fld qword ptr [C2]      {C2.real                                                        }
  fld qword ptr [C2+8]    {C2.imag    C2.real                                             }
  fld qword ptr [C1]      {C1.real    C2.imag   C2.real                                   }
  fld qword ptr [C1+8]    {C1.imag    C1.real   C2.imag    C2.real                        }

  fld ST(3)               {C2.real    C1.imag    C1.real   C2.imag   C2.real              }
  fmul ST(0), ST(0)       {C2.real²   C1.imag    C1.real   C2.imag   C2.real              }
  fld ST(3)               {C2.imag    C2.real²   C1.imag   C1.real   C2.imag    C2.real   }
  fmul ST(0), ST(0)       {C2.imag²   C2.real²   C1.imag   C1.real   C2.imag    C2.real   }
  faddp ST(1), ST(0)      {C2.i²+C2.r²C1.imag    C1.real   C2.imag   C2.real              }
  fld Smallest
  faddp ST(1), ST(0)
  fxch ST(4)              {C2.real    C1.imag    C1.real   C2.imag    divisor}

  fld ST(0)               {C2.real    C2.real    C1.imag    C1.real   C2.imag    divisor}
  fmul ST(0), ST(3)       {C2.r*C1.r  C2.real    C1.imag    C1.real   C2.imag    divisor}
  fld ST(2)               {C1.imag    C2.r*C1.r  C2.real   C1.imag    C1.real    C2.imag         divisor}
  fmul ST(0), ST(5)       {C1.i*C2.i  C2.r*C1.r  C2.real   C1.imag    C1.real    C2.imag         divisor}
  faddp ST(1), ST(0)      {a          C2.real    C1.imag    C1.real   C2.imag    divisor}
  fdiv ST(0), ST(5)       {result.r   C2.real    C1.imag    C1.real   C2.imag    divisor}
  fstp qword ptr [R] {C2.real    C1.imag    C1.real   C2.imag   divisor}

  fmulp ST(1), ST(0)      {C2.r*C1.i  C1.real    C2.imag   divisor}
  fxch ST(2)              {C1.real    C2.imag    C2.r*C1.i divisor}
  fmulp ST(1), ST(0)      {C1.r*C2.i  C2.r*C1.i  divisor}
  fsubp ST(1), ST(0)      {b          divisor}
  fdivrp ST(1), ST(0)     {result.i}
  fstp qword ptr [R+8]
end;
{$endif}
{$else}
inline;
var
   tmp: Double;
begin
     tmp:=C2.real*C2.real+C2.imag*C2.imag+Smallest;
     R.real:=(C1.real*C2.real+C1.imag*C2.imag)/tmp;
     R.imag:=(C2.real*C1.imag-C1.real*C2.imag)/tmp;
end;
{$endif}

procedure PPower(var C1, C2, R: TComplex);
{$ifdef UseAsm}
assembler;
asm                     {ST(0)  ST(1)  ST(2)  ST(3)  ST(4) }
  fld qword ptr [C1]    {C1.r                       }
  fld qword ptr [C1+8]  {C1.i   C1.r                       }
  fld LnOneHalf         {ln2/2    C1.i   C1.r                           }
  fld ST(2)             {C1.r   ln2    C1.i   C1.r                    }
  fmul ST(0),ST(0)      {C1.r^2 ln2    C1.i   C1.r                    }
  fld ST(2)             {C1.i   C1.r^2 ln2    C1.i   C1.r             }
  fmul ST(0),ST(0)      {C1.i^2 C1.r^2 ln2    C1.i   C1.r             }
  fld  Smallest         {Small  C1.i^2 C1.r^2 ln2    C1.i   C1.r             }
  faddp ST(1), ST(0)    {S+i1^2 C1.r^2 ln2    C1.i   C1.r             }
  faddp ST(1), ST(0)    {modC1  ln2    C1.i   C1.r             }
  fyl2x                 {ln(r)  C1.i   C1.r             }
  fxch ST(2)        {C1.r   C1.i      ln(r)}
  fpatan            {argC1  ln(r)                      }

  fld qword ptr [C2]    {C2.r    argC1  ln(r)              }
  fld qword ptr [C2+8]  {C2.i    C2.r   argC1  ln(r)       }
  fld ST(1)             {C2.r    C2.i   C2.r   argC1  ln(r)       }
  fmul ST(0), ST(4)     {r2*lnr  C2.i   C2.r   argC1  ln(r)              }
  fld ST(1)             {C2.i    r2*lnr  C2.i   C2.r   argC1  ln(r) }
  fmul ST(0), ST(4)     {i2*arg  r2*lnr C2.i   C2.r argC1  ln(r)       }
  fsubp ST(1), ST(0)    {a       C2.i   C2.r   argC1  ln(r)              }

  fxch ST(4)            {ln(r)   C2.i   C2.r   argC1  a              }
  fmulp ST(1), ST(0)    {lnr*i2  C2.r   argC1  a              }
  fxch ST(2)            {argC1   C2.r   lnr*i2 a              }
  fmulp ST(1), ST(0)    {r2*arg  lnr*i2 a              }
  faddp ST(1), ST(0)    {b       a}

  fsincos           {cos(b)  sin(b) a}
  fxch ST(1)
  fxch ST(2)        {a cos(b) sin(b) }

  fldl2e            {lg2(e) a      cos(b) sin(b)}
  fmulp st(1),st(0) {a*lg2e cos(b) sin(b)       }
  fld   st(0)       {x      x      cos(b) sin(b)       }
  frndint           {rnd(x) x      cos(b) sin(b)       }
  fsub  st(1),st(0) {rnd(x) frc(x) cos(b) sin(b)       }
  fxch              {frc(x) rnd(x) cos(b) sin(b)       }
  f2xm1             {2^st-1 rnd(x) cos(b) sin(b)       }
  fld1              {1      2^st-1 rnd(x) cos(b) sin(b)}
  faddp ST(1), ST(0)             {2^st   rnd(x) cos(b) sin(b)       }
  fscale            {2^x    rnd(x) cos(b) sin(b)       }
  fstp    st(1)     {exp(a) cos(b) sin(b)              }

  fmul ST(1), ST(0) {exp(a) Res.r  sin(b)              }
  fmulp ST(2),ST(0) {Res.r  Res.i                      }
  fstp  qword ptr [R]  {Res.i                             }
  fstp  qword ptr [R+8]
end;
{$else}
register;
begin
     R:=CExp(C2*CLn(C1));
end;
{$endif}

procedure PPowerR(var C1, C2, R: TComplex); register;
{$ifdef UseAsm}
assembler;
asm                     {ST(0)  ST(1)  ST(2)  ST(3)  ST(4) }
  fld qword ptr [C1]    {C1.r                       }
  fld qword ptr [C1+8]  {C1.i   C1.r                       }
  fld LnOneHalf         {ln2    C1.i   C1.r                           }
  fld ST(2)             {C1.r   ln2    C1.i   C1.r                    }
  fmul ST(0),ST(0)      {C1.r^2 ln2    C1.i   C1.r                    }
  fld ST(2)             {C1.i   C1.r^2 ln2    C1.i   C1.r             }
  fmul ST(0),ST(0)      {C1.i^2 C1.r^2 ln2    C1.i   C1.r             }
  fld  Smallest         {Small  C1.i^2 C1.r^2 ln2    C1.i   C1.r             }
  faddp ST(1), ST(0)    {S+i1^2 C1.r^2 ln2    C1.i   C1.r             }
  faddp ST(1), ST(0)    {modC1  ln2    C1.i   C1.r             }
  fyl2x                 {ln(r)  C1.i   C1.r             }
  fxch ST(2)            {C1.r   C1.i      ln(r)}
  fpatan                {argC1  ln(r)                      }

  fld qword ptr [C2]    {C2.r    argC1  ln(r)              }
  fld ST(0)             {C2.r    C2.r   argC1  ln(r)       }
  fmul ST(0), ST(3)     {r2*lnr  C2.r   argC1  ln(r)              }

  fxch ST(3)            {ln(r)   C2.r   argC1  a              }
  fstp ST(0)            {C2.r    argC1  a              }
  fmulp ST(1), ST(0)    {b       a              }

  fsincos           {cos(b)  sin(b) a}
  fxch ST(1)
  fxch ST(2)        {a cos(b) sin(b) }

  fldl2e            {lg2(e) a      cos(b) sin(b)}
  fmulp st(1),st(0) {a*lg2e cos(b) sin(b)       }
  fld   st(0)       {x      x      cos(b) sin(b)       }
  frndint           {rnd(x) x      cos(b) sin(b)       }
  fsub  st(1),st(0) {rnd(x) frc(x) cos(b) sin(b)       }
  fxch              {frc(x) rnd(x) cos(b) sin(b)       }
  f2xm1             {2^st-1 rnd(x) cos(b) sin(b)       }
  fld1              {1      2^st-1 rnd(x) cos(b) sin(b)}
  faddp ST(1), ST(0)             {2^st   rnd(x) cos(b) sin(b)       }
  fscale            {2^x    rnd(x) cos(b) sin(b)       }
  fstp    st(1)     {exp(a) cos(b) sin(b)              }

  fmul ST(1), ST(0) {exp(a) Res.r  sin(b)              }
  fmulp ST(2),ST(0) {Res.r  Res.i                      }
  fstp  qword ptr [r]  {Res.i                             }
  fstp  qword ptr [r+8]
end;
{$else}
register;
begin
     R:=CExp(C2.real*CLn(C1));
end;
{$endif}

procedure PSqr(var C1, C2, R: TComplex);
{$ifdef UseAsm}
assembler;
{$ifdef UseSSE}
asm
  movddup xmm0, [C1]    {a|a}
  movddup xmm1, [C1+8]  {a|a    b|b}
  movapd xmm2, xmm0     {a|a    b|b     a|a   }
  movapd xmm3, xmm1     {a|a    b|b     a|a   b|b}

  unpcklpd xmm2, xmm1    {a|a    b|b     b|a   b|b}
  unpcklpd xmm3, xmm0    {a|a    b|b     b|a   a|b}
  mulpd xmm0, xmm2      {ab|aa  b|b     b|a   a|b}
  mulpd xmm1, xmm3      {ab|aa  ab|bb   b|a   a|b}
  addsubpd xmm0, xmm1   {2ab|aa-bb  ab|bb   b|a   a|b}

  movupd [R], xmm0
end;
{$else}
asm
  fld qword ptr [C1]
  fld qword ptr [C1+8]
  fld ST(1)
  fmul ST(0),ST(0)
  fld ST(1)
  fmul ST(0),ST(0)
  fsubp ST(1), ST(0)
  fxch ST(2)

  fmulp ST(1), ST(0)
  fadd  ST(0), ST(0)
  fstp qword ptr [R+8]
  fstp qword ptr [R]
end;
{$endif}
{$else}
begin
     R.real:=C1.real*C1.real-C1.imag*C1.imag;
     R.imag:=2*C1.real*C1.imag;
end;
{$endif}

procedure PAbs(var C1, C2, R: TComplex);
{$ifdef UseAsm}
assembler;
asm
  fld qword ptr [C1]
  fmul ST(0),ST(0)
  fld qword ptr [C1+8]
  fmul ST(0),ST(0)
  faddp ST(1), ST(0)
  fsqrt
  fstp qword ptr [R]
end;
{$else}
register;
begin
     R.real:=sqrt(C1.real*C1.real+C1.imag*C1.imag);
end;
{$endif}

procedure PMod(var C1, C2, R: TComplex); register;
begin
     R.real:=C1.real*C1.real+C1.imag*C1.imag;
end;

procedure PNeg(var C1, C2, R: TComplex); register;
begin
     R.real:=-C1.real;
     R.imag:=-C1.imag;
end;

procedure PDouble(var C1, C2, R: TComplex); register;
{$ifdef UseAsm}
assembler;
{$ifdef UseSSE}
asm
  movupd xmm0, [C1]
  addpd xmm0, xmm0
  movupd [R], xmm0
end;
{$else}
asm
  fld qword ptr [C1]
  fadd ST(0), ST(0)
  fstp qword ptr [R]
  fld qword ptr [C1+8]
  fadd ST(0), ST(0)
  fstp qword ptr [R+8]
end;
{$endif}
{$else}
begin
     R.real:=C1.real+C1.real;
     R.imag:=C1.imag+C1.imag;
end;
{$endif}

procedure PLn(var C1, C2, R: TComplex); register;
begin
     R.real:=ln(CAbs(C1));
     R.imag:=CArg(C1);
end;

procedure PExp(var C1, C2, R: TComplex); register;
var
   rg: double;
begin
     rg:=fpcReplacementExp(C1.real);
     R.real:=rg*cos(C1.imag);
     R.imag:=rg*sin(C1.imag);
end;

procedure ReturnZ(var C1, C2, R: TComplex); register;
begin
  //C1 := ActiveZ;
end;

procedure ReturnC(var C1, C2, R: TComplex); register;
begin
  //C1 := ActiveC;
end;

function IsOperator(p: char): integer;
var
  i: integer;
begin
  Result := High(Operators) + 1;
  for i := 0 to High(Operators) do
    if p = Operators[i] then
    begin
      Result := i;
      Exit;
    end;
end;

function IsFunction(s: string; pos: integer): boolean;
var
  i: integer;
begin
  Result := false;
  for i := 0 to High(Functions) do
  begin
    if Copy(Functions[i], 1, Length(Functions[i]) - 1) = Copy(s, pos, Length(Functions[i]) - 1) then
    begin
      Result := true;
      Exit;
    end;
  end;
end;

function IsComplexNumber(s: string; Pos: integer): boolean;
begin
  if (s[Pos] in ['0'..'9']) or
    ( (Pos=1) and (s[Pos] in ['+','-']) and
      (Pos+1 <= Length(s)) and (s[Pos+1] in ['0'..'9']) ) or
    ( (Pos>1) and (s[Pos] in ['+','-']) and (s[Pos-1] in ['*','^','/','(']) and
      (Pos+1<=Length(s)) and (s[Pos+1] in ['0'..'9']) ) or
    ( (Pos+2<=Length(s)) and (s[Pos] in ['i']) and (s[Pos+1] in ['*']) ) then
    Result := true
  else Result := false;
end;

function ReadComplexNumber(s: string; sPos: integer; var ePos: integer): TComplex;
var
  i, j: integer;
  RPart, IPart: Double;
  IsRPart, IsIPart: boolean;
  Sign: integer;
  tmp: string;
label Again;
begin
IsIPart := false;
IsRPart := false;
Sign := 1;
IPart := 0;
RPart := 0;

i := sPos;
Again:

if s[i] in ['+', '-'] then
begin
  if s[i]='-' then sign := -1
  else sign := 1;
  inc(i);
  sPos := i;
end;

if s[i] = 'i' then
begin
  IsIPart := true;
  inc(i);
  if s[i] = '*' then inc(i);
  sPos := i;
end;

while (i <= Length(s)) and (s[i] in ['0'..'9', '.']) do
begin
  inc(i);
  if (i <= Length(s)) and (s[i] in ['e', 'E']) then
  begin
    inc(i);
    if (i <= Length(s)) and (s[i] in ['+', '-']) then inc(i);
    while (i <= Length(s)) and (s[i] in ['0'..'9']) do inc(i);
    break;
  end;
end;

ePos := i;

if (s[i] = '*') and (i+1 <= Length(s)) and (s[i+1]='i') then inc(ePos);
if s[ePos]='i' then
begin
  isIPart:=true;
  inc(ePos);
end;

if IsIPart then IPart := sign * StrToFloat(Copy(s,sPos,i-sPos))
else begin
  RPart := sign * StrToFloat(Copy(s,sPos,i-sPos));
  IsRPart := true;
end;

i := ePos;

j := GetNextOperator(s, i);
if (j < Length(s)) and (IsOperator(s[j])<2) then

if not (IsRPart and IsIPart) and
  not ( (sPos>1) and (s[sPos-1] in ['*', '^', '/']) ) and
  //falls der erste Teil der Zahl nicht mit einem Operator mit
  //Vorrang verknüpft ist
 (i < Length(s)) and (IsComplexNumber(s, i+1)) then
  //prüfe ob überhaupt ein zweiter Teil der Zahl existiert
  goto Again;

Result := MakeComplex(RPart, IPart);

end;

function GetClosingToken(s: string; sPos: integer): integer;
var
  OpenTokens: integer;
begin
  //s[sPos] = OpeningToken wird vorrausgesetzt
  OpenTokens := 1;
  Result := sPos;
  while OpenTokens > 0 do
  begin
    inc(Result);
    if Result > Length(s) then RaiseError($F0, Result - 1);
    if (s[Result] = ClosingToken) then dec(OpenTokens)
    else if (s[Result] = OpeningToken) then inc(OpenTokens);
  end;
end;

function GetNextOperator(s: string; sPos: integer): integer;
begin
  if s[sPos] = OpeningToken then
      Result := GetClosingToken(s, sPos)
  else Result := sPos+1; //der nächste Operator, d. h. > sPos wird gesucht
  while (Result <= Length(s)) and not (s[Result] in ['+','-','/','*','^']) do
  begin
    if s[Result] = OpeningToken then
      Result := GetClosingToken(s, Result);
    //nur Operatoren auf der gleichen Ebene, d.h. Klammern werden übersprungen
    inc(Result);
  end;
end;

end.

