unit UTFractal;

interface
uses UTComplex, SysUtils, Classes, Graphics, Controls, Forms, StdCtrls, ExtCtrls, Menus,
  ExtDlgs, Dialogs, LResources, UTRasterizer, UTCParser, UMisc, UTPixel, math;

const Length_Inner_ColorTable = 254;
      Length_Outer_ColorTable = 4096;

//Ein Punkt des Fraktals
type
  TCol = record
    a: Integer; //Periode
    i: Integer; //Iterationen
    le: Double; //Lyapunov-Exponent
    di: Double; //Farbwert
    Z: TComplex;
    C: TComplex;
    calculated: boolean;
  end;

type
  TTraceData = record
    C: TComplex;
    iterations: Integer;
    Z: array of TComplex;
    LyaSum: array of Double;
    LyaPartSums: array of Double;
    period: integer;
  end;

type
  TFractal = class(TObject)
  protected

  //++++++++++Datenverwaltung++++++++++
    data: array of array of TCol;  //enthält faktisch das ganze Fraktal
    procedure InitializeData;      //setzt data zurück
    function GetMArea: Double; //berechnet Fläche stabiler Punkte

  //++++++++++Lageparameter++++++++++
    Changed: boolean;
    //bei true muss das Fraktal neuberechnet werden
    Width, Height: extended;
    Top, Left, Bottom, Right: extended;
    Center: TComplex;
    Relation: Extended;            //Verhältnis X zu Y

  //++++++++++Zeichenparameter++++++++++
    Colorchanged: boolean;
    //bei true müssen die Farben neugezeichnet werden
    LastOuterValid, LastInnerValid: integer;
    //der letzte Farbmodus für den berechnet wurde
    SizeX, SizeY: integer;         //Größe in Pixeln
    StepX, StepY: Extended;        //Abstand zweier Pixel
    OuterColormode: shortint;
    //0 - DEM; 1 - DEM/SW; 2 - DEM/DwellSW; 3 - ET; 4 - LN
    InnerColormode: shortint;
    //0 - InnerColor; 1 - Black;
    OuterColorDwellLimit: integer;
    //Abstandsfaktor bei DEM/DwellSW für Schwarzfärbung
    colrepeat: integer;
    //Wiederholungen des Farbverlaufs, alle außer DEM/DwellSW
    OuterColortable: array of TPixel;
    //Der äußere Farbverlauf
    InnerColorTable: array of TPixel;
    //Die innere Einfärbung nach Perioden

  //++++++++++Zeichenfunktionen++++++++++
    RasterizerClass: TRasterizerClass; //entscheidet welche Ausgabe genutzt wird
    Rasterizer: TRasterizer;         //Render-Thread
    function GetColor(X, Y: Integer): TPixel;
    //entscheidet zwischen Outer und Inner

    OuterColorFunction: TColorFunction;
    //außen - zeigt auf eine der folgenden:
      function ReturnOutsideExpSmooth(X, Y: Integer): TPixel;
      function ReturnOutsideET(X, Y: Integer): TPixel;
      function ReturnOutsideLNET(X, Y: Integer): TPixel;
      function ReturnWhite(X, Y: Integer): TPixel;
      function ReturnDEM(X, Y: Integer): TPixel;
      function ReturnDEMSW(X, Y: Integer): TPixel;

    InnerColorFunction: TColorFunction;
    //innen - zeigt auf eine der folgenden:
      function ReturnInside(X, Y: Integer): TPixel;
      function ReturnBlack(X, Y: Integer): TPixel;
      function ReturnInsideLyaSmooth(X, Y: Integer): TPixel;
      function ReturnInsideExpSmooth(X, Y: Integer): TPixel;
      function ReturnInsideExpSmoothARG(X, Y: Integer): TPixel;
      function ReturnInsideARG(X, Y: Integer): TPixel;

  //++++++++++Berechnungsparameter++++++++++
    MaxIter: integer;              //maximale Iterationen
    LyaUpperBorder: extended;      //maximaler Lyapunov-Exponent (nur wBorders)
    LyaLowerBorder: extended;      //minimaler Lyapunov-Exponent (nur wBorders)
    LyaDwell: Double;
    calculatingmode: integer;      //0 - wBorders; 1 - Boilout; 2 - exakt;

  //++++++++++Berechnungsfunktionen++++++++++
    //Parser
    function ReturnColorLyapunovPeriods(X, Y: integer): TPixel;
    function ReturnColorLyapunov(X, Y: integer): TPixel;
    function ReturnColorLyapunovPeriodsDEM(X, Y: integer): TPixel;
    function ReturnColorBoiloutPeriods(X, Y: integer): TPixel;
    function ReturnColorBoilout(X, Y: integer): TPixel;

  //++++++++++Periodenbestimmung++++++++++
    CycArray: array of boolean;
    CycleArray: array of Integer; //Dient zur Bestimmung der Periode
    Orbit: array of TComplex; //speichert den letzten Orbit
    minCycPoints: integer;
    //Minimale Anzahl von Übereinstimmungen bevor abgebrochen wird

  //++++++++++Parser/Formel++++++++++

    FormulaParser: TCParser;       //Enthält Iterationsfunktion
    DerivateParser: TCParser;      //Ableitung der "
    Derivate2Parser: TCParser;     //2. Ableitung der "
    DEMParser: TCParser;           //Enthält verkettete Ableitung über C
    CriticalParser: TCParser;       //gibt kritischen Punkt zurück = Z_0

  //++++++++++Properties++++++++++
    //write
    procedure SetMaxIter(MI: integer);
    procedure SetLyaUpperBorder(border: extended);
    procedure SetLyaLowerBorder(border: extended);
    procedure SetCenter(C: TComplex);
    procedure SetOuterColormode(mode: shortint);
    procedure SetInnerColormode(mode: shortint);
    procedure SetCalculatingmode(c: integer);
    procedure SetColrepeat(X: Integer);
    procedure SetOuterColortable(X: Integer; T: TPixel);
    procedure SetInnerColortable(X: Integer; T: TPixel);
    procedure SetMinCycPoints(x: integer);
    procedure SetOuterColorDwellLimit(X: Integer);
    procedure SetLyapunovDwell(D: Double);
    //read
    function GetLyapunov(X, Y: Integer): Double;
    function GetCalculated(X, Y: Integer): boolean;
    function GetPeriod(X, Y: Integer): integer;
    function GetDistance(X, Y: Integer): Double;
    function GetOuterColortable(X: Integer): TPixel;
    function GetInnerColortable(X: Integer): TPixel;
    function GetIterated(X, Y: integer): integer;
    function GetFormula: string;
    function GetCriticalPoint: string;
    function GetDerivateFormula: string;
    function GetDerivate2Formula: string;
    function GetDEMFormula: string;

  public

  //++++++++++Zeichnen und Rechnen++++++++++
    procedure Draw;                //zeichnet das Fraktal
    procedure StopDrawing;         //beendet Render-Thread
    procedure Validate;            //Colorchanged=true; changed=true;
    Sizechanged: boolean;    //bei true muss eine neue Textur generiert werden

  //++++++++++++++++Parsen++++++++++++++++++
    function SetFormula(s: string): boolean;  //parst Iterations-Funktion
    function SetCriticalPoint(s: string): boolean;  //parst kritischen Punkt

  //++++++++++Speichern und Laden+++++++++++
    procedure LoadFromFile(f: string);
    procedure SaveToFile(f: string);
    procedure LoadInnerColorTable(f: string);
    procedure SaveInnerColorTable(f: string);

  //++++++++++Konstruktoren / Destruktoren++++++++++
    constructor Create(F, T: TComplex; SX, SY, MI: Integer); overload;
    constructor Create(F: TFractal); overload;
    destructor Destroy; override;

  //++++++++++Maße setzen++++++++++
    procedure SetSize(F, T: TComplex; SX, SY: integer);
    procedure Zoom(factor: extended; P: TPoint); overload;
    procedure Zoom(factor: extended); overload;

  //++++++++++Hilfsfunktionen++++++++++
    function CompToPoint(Z: TComplex): TPoint;
    function PointToComp(p: TPoint): TComplex;
    procedure TraceOrbit(var T: TTraceData);

  //++++++++++Properties++++++++++
    property Formula: string read GetFormula;
    property MArea: Double read GetMArea;
    property CritPoint: string read GetCriticalPoint;
    property DerivateFormula: string read GetDerivateFormula;
    property Derivate2Formula: string read GetDerivate2Formula;
    property DEMFormula: string read GetDEMFormula;
    property Width_pixels: integer read SizeX;
    property Height_pixels: integer read SizeY;
    property Width_real: extended read Width;
    property Height_real: extended read Height;
    property Top_real: extended read Top;
    property Left_real: extended read Left;
    property Right_real: extended read Right;
    property Bottom_real: extended read Bottom;
    property UpperBorder: extended read lyaUpperBorder write SetLyaUpperBorder;
    property LowerBorder: extended read lyaLowerBorder write SetLyaLowerBorder;
    property Iterations: integer read MaxIter write SetMaxIter;
    property Iterated[X, Y: integer]: integer read GetIterated;
    property Distance[X, Y: integer]: double read GetDistance;
    property Cen: TComplex read Center write SetCenter;
    property Outercolmode: shortint read Outercolormode write SetOutercolormode;
    property Innercolmode: shortint read Innercolormode write SetInnercolormode;
    property lyapunov[X, Y: integer]: Double read GetLyapunov;
    property Calculated[X,Y: integer]: boolean read GetCalculated;
    property period[X, Y: integer]: integer read GetPeriod;
    property Calcmode: integer read calculatingmode write SetCalculatingMode;
    property colorrepeat: integer read colrepeat write SetColRepeat;
    property OuterColTable[X: Integer]: TPixel read GetOuterColortable write SetOuterColorTable;
    property InnerColTable[X: Integer]: TPixel read GetInnerColortable write SetInnerColorTable;
    property minCyclePoints: integer read MinCycPoints write SetMinCycPoints;
    property OuterDwellLimit: integer read OutercolorDwellLimit write SetOuterColorDwellLimit;
    property LyapunovDwell: Double read LyaDwell write SetLyapunovDwell;
    property RasterizerThread: TRasterizerClass write RasterizerClass;
  end;

implementation

{###########################################################################
 |                          Datenverwaltung                                |
 ###########################################################################}
procedure TFractal.InitializeData;
//setzt den Datensatz des Fraktals zurück
var
  Point: TComplex;
  Line, Column: integer;
begin
  Point.imag := top;
  for Line := 0 to SizeY - 1 do //Setzen der Werte
  begin
    Point.real := left;
    for Column := 0 to SizeX - 1 do
    begin
      data[Line, Column].a := 0;
      data[Line, Column].le := 0;
      data[Line, Column].di := 0;
      data[Line, Column].i := 0;
      data[Line, Column].calculated := false;
      data[Line, Column].C := Point;
      Point.real := Point.real + StepX;
    end;
    Point.imag := Point.imag - StepY;
  end;
end;

function TFractal.GetMArea: Double;
var
  Line, Column: integer;
  stablePixels: integer;
begin
  Result := 0;
  stablePixels := 0;
  for Line := 0 to SizeY - 1 do //Setzen der Werte
    for Column := 0 to SizeX - 1 do
      if data[Line, Column].le <= 0 then inc(StablePixels);
  //Result := stablePixels * StepX * StepY;
  Result := (SizeY*SizeX-stablePixels) * StepX * StepY;
end;

{###########################################################################
 |                         Zeichenfunktionen                               |
 ###########################################################################}

function TFractal.GetColor(X, Y: Integer): TPixel; inline;
//ruft OuterColorFunction für alle _instabilen_ (le>Dwell) auf
//InnerColorFunction für den Rest
begin
     if (data[Y, X].le >= LyaDwell) then Result := OuterColorFunction(X, Y)
     else Result := InnerColorFunction(X, Y);
end;

{###########################Außen#########################################}
function TFractal.ReturnOutsideExpSmooth(X, Y: Integer): TPixel;
var
  i: integer;
  f: Double;
begin
  if changed then //Falls neuberechnet wurde, den Farbwert neugenerieren
  begin
    f := 0;
    for i := 0 to data[Y,X].i do
      f := f + fpcReplacementExp( - CModule( Orbit[i] ) );
    data[Y,X].di := f / MaxIter;
  end;
  i := round(sqrt(data[Y,X].di/e*2) * colorrepeat * Length_Outer_ColorTable);
  if ((i div Length_Outer_ColorTable) mod 2 = 0) then
    Result := OuterColorTable[ i mod Length_Outer_ColorTable]
  else Result := OuterColorTable[ Length_Outer_ColorTable -
    (i mod Length_Outer_ColorTable) ];
end;

function TFractal.ReturnOutsideET(X, Y: Integer): TPixel;
var
   iter: integer;
begin
  iter:=round(data[Y,X].i / MaxIter*colorrepeat*Length_Outer_ColorTable);
  if (iter div Length_Outer_ColorTable) mod 2 = 0 then
  Result := OuterColorTable[ iter  mod Length_Outer_ColorTable]
  else Result := OuterColorTable[ Length_Outer_ColorTable - iter  mod Length_Outer_ColorTable];
end;

function TFractal.ReturnOutsideLNET(X, Y: Integer): TPixel;
var
   iter: integer;
   iterf: double;
begin
  iterf := data[Y,X].i + 1 - abs(ln(ln(CAbs(data[Y,X].Z)+1)+1)/ln(5));
  iter:=round(iterf / MaxIter*colorrepeat*Length_Outer_ColorTable);
  if (iter div Length_Outer_ColorTable) mod 2 = 0 then
  Result := OuterColorTable[ iter  mod Length_Outer_ColorTable]
  else Result := OuterColorTable[ Length_Outer_ColorTable - iter  mod Length_Outer_ColorTable];
end;

function TFractal.ReturnWhite(X,Y: Integer): TPixel;
begin
  Result := ClWhite32;
end;

{###########################Innen#########################################}
function TFractal.ReturnInside(X, Y: Integer): TPixel;
begin
  if data[Y, X].a = 0 then Result := ClBlack32
  else Result := InnerColorTable[(data[Y, X].a - 1) mod (Length_Inner_ColorTable)]
end;

function TFractal.ReturnBlack(X, Y: Integer): TPixel;
begin
  Result := ClBlack32;
end;

function TFractal.ReturnInsideExpSmooth(X, Y: Integer): TPixel;
var
  i, a: integer;
  f: Double;
begin
  if changed = true then
  begin
    a := data[Y,X].a;
    if a > 0 then
    begin
      i := data[Y,X].i - a;
      f := 0;
      while i >=0 do
      begin
        f := f + fpcReplacementExp(
          -  1 / (CModule( (Orbit[i+a]-Orbit[i]) ) + Smallest) );
        dec(i);
      end;
     data[Y,X].di := f / MaxIter / CAbs(Orbit[0]-Orbit[data[Y,X].i]);
    end else data[Y,X].di := 0;
  end;
  i := round(sqrt(data[Y,X].di/e*2) * colorrepeat * Length_Outer_ColorTable);
  if ((i div Length_Outer_ColorTable) mod 2 = 0) then
    Result := OuterColorTable[ i mod Length_Outer_ColorTable]
  else Result := OuterColorTable[ Length_Outer_ColorTable -
    (i mod Length_Outer_ColorTable) ];
end;

function TFractal.ReturnInsideLyaSmooth(X, Y: Integer): TPixel;
var
  iter: integer;
  iterf: Double;
begin
  iterf := 1/(abs(data[Y,X].le)+1);
  iter := round(sqrt(iterf) * colorrepeat * Length_Outer_ColorTable);
  if ((iter div Length_Outer_ColorTable) mod 2 = 0) then
    Result := OuterColorTable[ iter mod Length_Outer_ColorTable]
  else Result := OuterColorTable[ Length_Outer_ColorTable -
    (iter mod Length_Outer_ColorTable) ];
end;

function TFractal.ReturnInsideExpSmoothARG(X, Y: Integer): TPixel;
var
  i, a: integer;
  f: Double;
  h,s,l: single;
begin
  if changed = true then
  begin
    a := data[Y,X].a;
    if a > 0 then
    begin
      a := data[Y,X].a;
      i := data[Y,X].i - a;
      f := 0;
      while i >=0 do
      begin
        f := f + fpcReplacementExp(
          -  1 / (CModule( (Orbit[i+a]-Orbit[i]) ) + Smallest) );
        dec(i);
      end;
     data[Y,X].di := f / MaxIter / CAbs(Orbit[0]-Orbit[data[Y,X].i]);
    end else data[Y,X].di := 0;
  end;
  h := (CArg(data[Y,X].Z)+pi)/2/pi;
  s := 1;
  f := sqrt(data[Y,X].di/e*2)*colorrepeat;
  i := trunc(f);
  if i mod 2 = 1 then f := f - i
  else f := 1 - (f - i);
  l := 1 - 0.8 * f;
  Result := HSLToPixel(h,s,l);
end;

function TFractal.ReturnInsideARG(X, Y: Integer): TPixel;
var
  iterf: Double;
begin
  iterf := (CArg(data[Y,X].Z)+pi)/2/pi;
  Result := HSLToPixel(iterf,0.8,1);
end;

function TFractal.ReturnDEM(X, Y: Integer): TPixel;
var
  //iter, rep: integer;
  tmp, tmp2: Double;
begin
  if abs(data[Y,X].di/4)<sqr(StepX+StepY)/4 then tmp := 1
  else tmp := abs(ln(abs(data[Y,X].di/4)))/OuterColorDwellLimit;
  tmp2 := colorrepeat * tmp;
  if floor(tmp2) mod 2 = 0 then tmp := frac(tmp2)
  else tmp := 1 - frac(tmp2);
  tmp := tmp * Length_Outer_ColorTable;
  if tmp<0 then tmp:=0 else if tmp>Length_Outer_ColorTable then tmp:=Length_Outer_ColorTable;
  Result:=Outercolortable[floor(tmp)];
end;


function TFractal.ReturnDEMSW(X, Y: Integer): TPixel;
var
  a: single;
  b: byte;
begin
    a:=abs(data[Y,X].di)/(StepX+StepY)*colorrepeat*255;
    if a>255 then a:=255;
    b:=byte(round(a));
    Result:=RGBtoPixel(b,b,b);
end;


{###########################################################################
 |                       Berechnungsfunktionen                             |
 ###########################################################################}

{###########################Allgemein#########################################}

function TFractal.ReturnColorLyapunovPeriods(X, Y: integer): TPixel;
var
  i, j: integer; {Zähler}
  dist, le: Double;
  O: TCPointer;
  D: ^TCol;
  FirstStable, CurCyc, CurPoint, CycStart: integer;
  label ende;
begin
  D := @data[Y,X];    {Zeiger auf aktuellen Datensatz}
  O := @Orbit[0];     {Zeiger auf das Orbit-Array}

  dist := lyaLowerBorder;
  FirstStable := -1;
  {Lyapunov-Summe noch nicht unter LowerBorder gefallen}

  CriticalParser.C := D^.C;
  FormulaParser.C := CriticalParser.C;
  FormulaParser.Z := CriticalParser.Evaluate;
  DerivateParser.C := CriticalParser.C;
  DerivateParser.Z := FormulaParser.Z;
  {Initialisieren der Parser-Objekte}
  {FormulaParser = Iterationsfunktion}
  {DerivateParser = Ableitung der Iterationsfunktion}
  {CriticalParser = Gibt Anfangswert der Iteration zurück}

  O^ := FormulaParser.Z;

  le := 0; {Lyapunov-Summe}
  i := 1;
  while (i < MaxIter) do
  begin
    DerivateParser.Z:=FormulaParser.IterateZ;
    le := le + ln ( CModule ( DerivateParser.Evaluate ) );// + 1E-10);

    inc(O);
    O^ := FormulaParser.Z;
    {i-te Iteration wird gespeichert}

    if le > lyaUpperBorder then break {instabil}
    else if (le < dist) then {stabil}
    begin
      {Bestimmung der Periode}
      CycArray[i] := true;
      dist := le;
      if FirstStable > -1 then
      begin
        for CurCyc := (i - FirstStable) div minCycPoints downto 1 do
          for CycStart := FirstStable to FirstStable + CurCyc do
          begin
            CurPoint := CycStart;
            while (CurPoint <= i) and (CycArray[CurPoint]=true) do
              inc(CurPoint, CurCyc);
            if CurPoint > i then
            begin
              D^.a := CurCyc; {Periode speichern}
              goto ende; {Periode gefunden, deshalb abbrechen}
            end;
          end;
      end else FirstStable := i;
    end else CycArray[i] := false; {noch nicht klar}
  inc(i);
  end;
  D^.a := 0;
ende:
  D^.i := i; {Iteration speichern}
  D^.le := le / i / 2; {Lyapunov-Exponent berechnen}
  D^.Z := FormulaParser.Z; {Letzten Wert der Iteration speichern}
  Result := GetColor(X,Y); {Farbwert anfordern}
  D^.calculated := true;
  {vermittelt dem Rasterizer-Objekt, das dieser Pixel berechnet wurde}
end;

function TFractal.ReturnColorLyapunov(X, Y: integer): TPixel;
var
  tmpCol: TCol;
  tmp: Double;
begin

  tmpcol := data[Y, X];
  CriticalParser.C := tmpcol.C;
  FormulaParser.C := tmpcol.C;         //Hauptiteration;
  FormulaParser.Z := CriticalParser.Evaluate;
  DerivateParser.C := tmpcol.C;         //Ableitung der Iteration
  DerivateParser.Z := FormulaParser.Z;

  while (tmpCol.i < MaxIter div 2) do
  begin
    FormulaParser.IterateZ;
    if CModule(FormulaParser.Z) > 1E100 then
    begin
      tmpCol.le := MaxIter;
      tmpCol.i := MaxIter;
      break;
    end;
    inc(tmpCol.i);
  end;

  while (tmpCol.i < MaxIter) do
  begin
    DerivateParser.Z:=FormulaParser.IterateZ;
    tmp := CModule(DerivateParser.Evaluate);
    if CModule(FormulaParser.Z) > 1E100 then
    begin
      tmpCol.le := MaxIter;
      tmpCol.i := MaxIter;
      break;
    end;
    tmpCol.le += ln(tmp);
    inc(tmpCol.i);
  end;

  tmpcol.le /= (tmpcol.i + 1) * 2;
  tmpcol.Z := FormulaParser.Z;
  data[Y, X] := tmpCol;

  if (tmpCol.le <= 1E-2) then Result := ClBlack32
  else Result := ClWhite32;

end;

function TFractal.ReturnColorLyapunovPeriodsDEM(X, Y: integer): TPixel;
var
  tmpCol: TCol;
  dist: Double;
//++++++ Periodizitäts-Prüfung +++++++
  CycPoints,  iter, maxCyc: integer;
  OldCycPoint, NewCycPoint, diff: integer;
  PointsOnCycle: integer;
label ende;
begin
  CycPoints := 0;
  PointsOnCycle := 1;
  maxCyc := 1;
  OldCycPoint := 0;
  NewCycPoint := 1;
  dist := lyaLowerBorder;

  tmpcol := data[Y, X];
  tmpCol.di := 1;
  CriticalParser.C := tmpcol.C;
  FormulaParser.C := tmpcol.C;         //Hauptiteration;
  FormulaParser.Z := CriticalParser.Evaluate;
  DerivateParser.C := tmpcol.C;         //Ableitung der Iteration
  DerivateParser.Z := FormulaParser.Z;
  DEMParser.C := tmpCol.C;
  while (tmpCol.i < MaxIter) do
  begin
    DEMParser.Z:=DEMParser.Evaluate*FormulaParser.Z;
    FormulaParser.IterateZ;
    DerivateParser.Z:=FormulaParser.Z;
    tmpCol.le += ln(CModule(DerivateParser.Evaluate)+Smallest);

    if tmpCol.le < dist then
    begin
      CycleArray[CycPoints] := tmpCol.i;
      if (CycPoints >= MinCycPoints) and
        (tmpCol.i >= maxCyc * MinCycPoints) then
        while (CycPoints - NewCycPoint >= MinCycPoints - PointsOnCycle) do
        begin
          diff := CycleArray[NewCycPoint] - CycleArray[OldCycPoint];
          if diff > maxCyc then
          begin
            maxCyc := diff;
            OldCycPoint := 0;
            NewCycPoint := 1;
            PointsOnCycle := 1;
            continue;
          end else if diff = maxCyc then
            if PointsOnCycle < MinCycPoints then
            begin
              PointsOnCycle += 1;
              OldCycPoint := NewCycPoint;
            end else begin
              tmpCol.a := maxCyc;
              goto ende;
            end;
          NewCycPoint += 1;
        end;
      CycPoints += 1;
      dist := tmpCol.le;
    end else if tmpCol.le > lyaUpperBorder then break;

    inc(tmpCol.i);
  end;
ende:
  tmpcol.le /= (tmpcol.i + 1) * 2;
  tmpCol.di := ln(CModule(FormulaParser.Z))*CAbs(FormulaParser.Z)
    / CAbs(DEMParser.Z);
  tmpcol.Z := FormulaParser.Z;
  data[Y, X] := tmpCol;

  if tmpCol.le < 0 then
  begin
    {tmpcol.di := //(CArg(tmpCol.Z)+pi)/2/pi; //*
    tmpcol.i / MaxIter * 1/(abs(tmpcol.le)+1);}

    Result := InnerColorFunction(X, Y);
  end else begin
      //data[Y, X] := tmpCol;
      Result := OuterColorFunction(X, Y);
  end;
end;

function TFractal.ReturnColorBoiloutPeriods(X, Y: integer): TPixel;
var
  tmpCol: TCol;
  Zs: array of TComplex;
  h, i: integer;
begin
  SetLength(Zs, MaxIter);


  tmpcol := data[Y, X];
  tmpCol.le := -1;
  CriticalParser.C := tmpcol.C;
  FormulaParser.C := tmpcol.C;         //Hauptiteration;
  FormulaParser.Z := CriticalParser.Evaluate;
  DerivateParser.C := tmpcol.C;         //Ableitung der Iteration
  DerivateParser.Z := FormulaParser.Z;

  while (tmpCol.i < MaxIter) do
  begin
    DerivateParser.Z:=FormulaParser.IterateZ;
    Zs[tmpCol.i] := FormulaParser.Z;
    i := tmpCol.i;
    for h:= i-1 downto 0 do
      if CAbs(Zs[i] - Zs[h]) < 1E-15 then
      begin
        tmpCol.a := i - h;
        break;
      end;
    if CModule( FormulaParser.Z ) > LyaUpperBorder then
    begin
      tmpcol.le := 1;
      break;
    end;
    tmpCol.di += fpcReplacementExp( - CModule( FormulaParser.Z ) );
    inc(tmpCol.i);
  end;

  tmpcol.Z := FormulaParser.Z;
  tmpcol.di /= MaxIter;
  data[Y, X] := tmpCol;

  if tmpCol.le > 0 then Result := OuterColorFunction(X, Y)
  else Result := InnerColorFunction(X, Y);

end;

function TFractal.ReturnColorBoilout(X, Y: integer): TPixel;
var
  tmpCol: TCol;
  Zs: array of TComplex;
  i, h: integer;
begin

  SetLength(Zs, MaxIter);
  tmpcol := data[Y, X];
  tmpCol.le := -1;
  CriticalParser.C := tmpcol.C;
  FormulaParser.C := tmpcol.C;         //Hauptiteration;
  FormulaParser.Z := CriticalParser.Evaluate;
  DerivateParser.C := tmpcol.C;         //Ableitung der Iteration
  DerivateParser.Z := FormulaParser.Z;

  while (tmpCol.i < MaxIter) do
  begin
    DerivateParser.Z:=FormulaParser.IterateZ;
    Zs[tmpCol.i] := FormulaParser.Z;
    if CModule( FormulaParser.Z ) > LyaUpperBorder then
    begin
      tmpcol.le := 1;
      break;
    end;
    tmpCol.di += fpcReplacementExp( - CModule( FormulaParser.Z ) );
    inc(tmpCol.i);
  end;

  if tmpCol.le <= 0 then
  begin
    h := MaxIter - 1;
    for i := h-1 downto 0 do
      if CAbs(Zs[i] - Zs[h]) < 1E-15 then
      begin
        tmpCol.a := h - i;
        break;
      end;
  end;

  tmpcol.Z := FormulaParser.Z;
  tmpcol.di /= MaxIter;
  data[Y, X] := tmpCol;

  if tmpCol.le > 0 then Result := OuterColorFunction(X, Y)
  else Result := InnerColorFunction(X, Y);

end;

{###########################################################################
 |                               Parser                                    |
 ###########################################################################}

function TFractal.SetFormula(s: string): boolean;
var
   i: integer;
begin
     Result := FormulaParser.Parse(s);

     if not Result then exit;

     Changed:=true;
     Colorchanged:=true;
     {FG.mem_parseDebug.Clear;
     FG.mem_parseDebug.Lines.Add('Formel:');}
     FormulaParser.PrintTree;

     DerivateParser.Parse(s);
     DerivateParser.Differentiate;

     {FG.mem_parseDebug.Lines.Add('');
     FG.mem_parseDebug.Lines.Add('Ableitung:');}
     //DerivateParser.PrintTree;

     Derivate2Parser.Parse(s);
     Derivate2Parser.Differentiate;
     Derivate2Parser.Differentiate;

     for i:=1 to Length(s) do
         if (s[i] in ['c', 'C']) then s[i]:='Z';

     DEMParser.Parse(s);
     DEMParser.Differentiate;

     {FG.mem_parseDebug.Lines.Add('');
     FG.mem_parseDebug.Lines.Add('DEM:');}
     //DEMParser.PrintTree;
end;

function TFractal.SetCriticalPoint(s: string): boolean;
begin
     Result := CriticalParser.Parse(s);
     CriticalParser.PrintTree;
end;

{###########################################################################
 |                        Zeichnen und Rechnen                             |
 ###########################################################################}

procedure TFractal.Draw;
//Hier werden vor der Berechnung die Zeiger auf die Farb und Berechnungs-
//funktionen anhand der gewählten Parameter initialisiert.
//Dann wird der Render-Thread gestartet
begin
     StopDrawing;

     case OuterColorMode of
          0:  OuterColorFunction := @ReturnOutsideExpSmooth;
          1:  OuterColorFunction := @ReturnOutsideET;
          2:  OuterColorFunction := @ReturnOutsideLNET;
          3:  OuterColorFunction := @ReturnWhite;
          else OuterColorFunction := @ReturnOutsideExpSmooth;
     end;
     case InnerColorMode of
          0: InnerColorFunction := @ReturnInside; //Inside;
          1: InnerColorFunction := @ReturnBlack;
          2: InnerColorFunction := @ReturnInsideLyaSmooth;
          3: InnerColorFunction := @ReturnInsideExpSmooth;
          4: InnerColorFunction := @ReturnInsideExpSmoothARG;
          5: InnerColorFunction := @ReturnInsideARG;
          else InnerColorFunction := @ReturnBlack;
     end;
  if Changed = true then
  begin
    InitializeData;
    Rasterizer := RasterizerClass.Create(True);
    LastOuterValid := OuterColorMode;
    LastInnerValid := InnerColorMode;
    case calculatingmode of
         0: Rasterizer.ColorFunction := @ReturnColorLyapunovPeriods;
         1: Rasterizer.ColorFunction := @ReturnColorLyapunov;
         2: Rasterizer.ColorFunction := @ReturnColorBoiloutPeriods;
         3: Rasterizer.ColorFunction := @ReturnColorBoilout;
    end;
    Rasterizer.Resume;
  end else if Colorchanged = true then
  begin
    Rasterizer := RasterizerClass.Create(True);
    Rasterizer.ColorFunction := @GetColor;
    Rasterizer.Mode:=True;
    Rasterizer.Resume;
  end;
end;

procedure TFractal.StopDrawing;
begin
     if Rasterizer<>nil then
     begin
          Rasterizer.Terminate;
          //while Rasterizer<>nil do sleep(250);
          sleep(250);
     end;
end;

procedure TFractal.Validate;
begin
     Colorchanged:=false;
     Changed:=false;
end;

procedure TFractal.SaveToFile(f: string);
var
  D: Text;
begin
  Assign(D, f);
  Rewrite(D);

  WriteLn(D, FloatToStr(Top));
  WriteLn(D, FloatToStr(Left));
  WriteLn(D, FloatToStr(Right));
  WriteLn(D, FloatToStr(Bottom));
  WriteLn(D, IntToStr(SizeX));
  WriteLn(D, IntToStr(SizeY));
  WriteLn(D, IntToStr(MaxIter));
  WriteLn(D, FormulaParser.ParsedString);
  WriteLn(D, CriticalParser.ParsedString);

  Close(D);
end;

procedure TFractal.LoadFromFile(f: string);
var
  D: Text;
  s: string;
  t,l,b,r: Double;
  sx,sy: integer;
begin
  Assign(D, f);
  Reset(D);

  ReadLn(D, s);
  t := StrToFloat(s);
  ReadLn(D, s);
  l := StrToFloat(s);
  ReadLn(D, s);
  r := StrToFloat(s);
  ReadLn(D, s);
  b := StrToFloat(s);
  ReadLn(D, s);
  sx := StrToInt(s);
  ReadLn(D, s);
  sy := StrToInt(s);
  SetSize(MakeComplex(l,t),MakeComplex(r,b),sx,sy);

  ReadLn(D, s);
  MaxIter := StrToInt(s);
  ReadLn(D, s);
  SetFormula(s);
  ReadLn(D, s);
  SetCriticalPoint(s);

  Close(D);
end;


procedure TFractal.LoadInnerColorTable(f: string);
var
  D: Text;
  s: string;
  i,j: integer;
  R,G,B: Byte;
begin
  Assign(D, f);
  Reset(D);
  i:=0;
  while not EoF(D) and (i<Length_Inner_ColorTable) do
  begin
    ReadLn(D, s);
    for j:=1 to 3 do
    begin
      case j of
        1: R:=byte(StrToInt(Copy(s, 1, Pos(' ', s)-1)));
        2: G:=byte(StrToInt(Copy(s, 1, Pos(' ', s)-1)));
        3: B:=byte(StrToInt(Copy(s, 1, Length(s))));
      end;
      Delete(s, 1, Pos(' ', s));
    end;
    InnerColorTable[i]:=RGBtoPixel(R,G,B);
    inc(i);
  end;

  for j := i to Length_Inner_ColorTable do
    InnerColorTable[j] := InnerColorTable[j - i];
  Close(D);
end;

procedure TFractal.SaveInnerColorTable(f: string);
var
  D: LongInt;
  i: integer;
begin
  D:=FileCreate(f);
  for i:=0 to Length_Inner_ColorTable do
    FileWrite(D, InnerColorTable[i], SizeOf(InnerColorTable[i]));
  FileClose(D);
end;

{###########################################################################
 |                       Konstruktoren / Destruktoren                      |
 ###########################################################################}

constructor TFractal.Create(F, T: TComplex; SX, SY, MI: Integer); overload;
var
   i: integer;
begin
  inherited Create;
  FormulaParser := TCParser.Create;
  DerivateParser := TCParser.Create;
  Derivate2Parser := TCParser.Create;
  DEMParser := TCParser.Create;
  CriticalParser := TCParser.Create;
  RasterizerClass := TOGLRasterizer;  //erzeugt die standardmäßige OGL-Ausgabe
  //Grundvariablen
  MaxIter := MI;
  SetLength(CycleArray, MaxIter+1);
  SetLength(CycArray, MaxIter+1);
  SetLength(Orbit, MaxIter+1);
  SetLength(InnerColorTable, Length_Inner_ColorTable+1);
  SetLength(OuterColorTable, Length_Outer_ColorTable+1);
  for i:=0 to High(InnerColorTable) do InnerColorTable[i]:=clBlack32;

  lyaUpperBorder := 100;
  lyaLowerBorder := -20;
  minCycPoints:=5;
  //Maße der komplexen Ebene
  Left := F.real;
  Top := F.imag;
  Right := T.real;
  Bottom := T.imag;
  //Hilfsvariablen
  Width := Right - Left;
  Height := Top - Bottom;
  Relation := Width / Height;
  Center := MakeComplex(Width / 2 + Left, Height / 2 + Bottom);
  //Variablen zum Zeichnen des Fraktals
  SizeX := SX;
  SizeY := SY;
  SizeChanged:=true;
  Outercolormode := 0;
  colorrepeat:=1;
  //Seitenverhältnis anpassen
  if (SX / SY <> Relation) then SizeY := round(SX / Relation);
  StepX := Width / SizeX;
  StepY := Height / SizeY;
  //Datensatz initialisieren
  calculatingmode := 0;
  SetLength(data, SizeY, SizeX);
  InitializeData;
   //Sonstiges
  Colorchanged := true;
  Changed := true;
  OuterColorDwellLimit:=10;
end;

constructor TFractal.Create(F: TFractal); overload;
begin
  inherited Create;
  Left := F.Left;
  Top := F.Top;
  Right := F.Right;
  Bottom := F.Bottom;
  Width := F.Width;
  Height := F.Height;
  Relation := F.Relation;
  Center := F.Center;
  SizeX := F.SizeX;
  SizeY := F.SizeY;
  SizeChanged := F.SizeChanged;
  StepX := F.StepX;
  StepY := F.StepY;

  MaxIter := F.MaxIter;
  LyaUpperBorder := F.LyaUpperBorder;
  LyaLowerBorder := F.LyaLowerBorder;
  minCycPoints := F.minCycPoints;
  SetLength(CycleArray, MaxIter+1);
  CalculatingMode := F.CalculatingMode;
  data := Copy(F.data);

  OuterColorMode := F.OuterColorMode;
  LastOuterValid := -1;
  LastInnerValid := -1;
  ColorRepeat := F.ColorRepeat;
  InnerColorTable := Copy(F.InnerColorTable);
  OuterColorTable := Copy(F.OuterColortable);
  OuterColorDwellLimit := F.OuterColorDwellLimit;

  Changed := F.Changed;
  ColorChanged := F.ColorChanged;

  RasterizerClass := F.RasterizerClass;
  //erzeugt die standardmäßige OGL-Ausgabe

  FormulaParser := TCParser.Create;
  DerivateParser := TCParser.Create;
  Derivate2Parser := TCParser.Create;
  DEMParser := TCParser.Create;
  CriticalParser := TCParser.Create;

  if F.Formula <> '' then SetFormula(F.Formula);
  if F.CritPoint <> '' then SetCriticalPoint(F.CritPoint);

end;

destructor TFractal.Destroy;
begin
   if Rasterizer<>nil then
   begin
     Rasterizer.Terminate;
     //WaitForThreadTerminate(Rasterizer.Handle, 5000);
     sleep(50);
     //Rasterizer.Free;
   end;
     FormulaParser.ClearTree;
     DerivateParser.ClearTree;
     CriticalParser.ClearTree;
     inherited;
end;

{###########################################################################
 |                              Maße setzen                                |
 ###########################################################################}

procedure TFractal.SetSize(F, T: TComplex; SX, SY: integer);
begin
  if not (F=MakeComplex(Left, Top)) or
    not (T=MakeComplex(Right, Bottom)) or
    not (SX = SizeX) or not (SY = SizeY) then
  begin
  //Maße der komplexen Ebene
    Left := F.real;
    Top := F.imag;
    Right := T.real;
    Bottom := T.imag;
  //Hilfsvariablen
    Width := Right - Left;
    Height := Top - Bottom;
    Relation := Width / Height;
    Center := MakeComplex(Width / 2 + Left, Height / 2 + Bottom);
  //Variablen zum Zeichnen des Fraktals
    if (SX<>SizeX) or (SizeY<>SY) then
    begin
         SizeChanged:=true;
         SizeX := SX;
         SizeY := SY;
    end;
    //Seitenverhältnis anpassen
    if round(SX / Relation) <> SizeY then
    begin
      SizeChanged:=true;
      SizeY := round(SX / Relation);
    end;
    StepX := Width / SizeX;
    StepY := Height / SizeY;
   //Datensatz initialisieren
    SetLength(data, SizeY, SizeX);
    InitializeData;
    Changed := true;
  end;
end;

procedure TFractal.SetCenter(C: TComplex);
begin
  if not (Center=C) then
  begin
    Center := C;
    Changed := true;
  end;
end;

procedure TFractal.Zoom(factor: extended; P: TPoint);
begin
  Center := PointToComp(P);
  Left := Center.real - Width / 2 / factor;
  Top := Center.imag + Height / 2 / factor;
  Width := Width / factor;
  Height := Height / factor;
  Right := Left + Width;
  Bottom := Top - Height;
  StepX := Width / SizeX;
  StepY := Height / SizeY;
  changed := true;
end;

procedure TFractal.Zoom(factor: extended);
begin
  Left := Center.real - Width / 2 / factor;
  Top := Center.imag + Height / 2 / factor;
  Width := Width / factor;
  Height := Height / factor;
  Right := Left + Width;
  Bottom := Top - Height;
  StepX := Width / SizeX;
  StepY := Height / SizeY;
  changed := true;
end;

{###########################################################################
 |                              Hilfsfunktionen                            |
 ###########################################################################}

function TFractal.CompToPoint(Z: TComplex): TPoint;
begin
 if ((Z.real > right) or (Z.real < left) or
    (Z.imag > top) or (Z.imag < bottom)) and (CAbs(Z)>50) then
  begin
    Result.X := -1;
    Result.Y := -1;
  end else
  begin
    Result.X := round(abs(Z.real - Left) / StepX);
    Result.Y := round(abs(Top - Z.imag) / StepY);
  end;
end;

function TFractal.PointToComp(p: TPoint): TComplex;
begin
  if (P.X >= 0) and (P.X < SizeX) and (P.Y >= 0) and (P.Y < SizeY) then
     Result:=data[P.Y,P.X].C
  else Result := 0;
end;

procedure TFractal.TraceOrbit(var T: TTraceData);
var
  i: integer;
  dist, le: Double;
//++++++ Periodizitäts-Prüfung +++++++
  CycPoints, maxCyc: integer;
  OldCycPoint, NewCycPoint, diff: integer;
  PointsOnCycle: integer;
label ende;
begin
  Iterations := T.iterations;
  CycPoints := 0;
  PointsOnCycle := 1;
  maxCyc := 1;
  OldCycPoint := 0;
  NewCycPoint := 1;
  dist := lyaLowerBorder;

  CriticalParser.C := T.C;
  FormulaParser.C := T.C;         //Hauptiteration;
  DerivateParser.C := T.C;         //Ableitung der Iteration
  FormulaParser.Z := CriticalParser.Evaluate;
  DerivateParser.Z := FormulaParser.Z;

  SetLength(T.Z,T.iterations);
  SetLength(T.LyaSum, T.iterations);
  SetLength(T.LyaPartSums, T.iterations);
{for i:=0 to T.iterations-1 do
begin
  T.Z[i] := 0;
  T.LyaSum[i] := 0;
  T.LyaPartSums[i] := 0;
end;}
  T.period := 0;
  le := 0;
  i := 0;
  while (i < T.iterations) do
  begin
    T.Z[i] := FormulaParser.Z;
    if i<>0 then //kritischen punkt mit ln(c) = -inf ausschließen
    T.LyaSum[i] := ln(CModule(DerivateParser.Evaluate)) / 2 / T.iterations;
    le += T.LyaSum[i]; //ln(CModule(DerivateParser.Evaluate));
    T.LyaPartSums[i] := le;

    if (le < dist) and (T.period = 0) then
    begin
      CycleArray[CycPoints] := i;
      if (CycPoints >= MinCycPoints) and
        (i >= maxCyc * MinCycPoints) then
        while (CycPoints - NewCycPoint >= MinCycPoints - PointsOnCycle) do
        begin
          diff := CycleArray[NewCycPoint] - CycleArray[OldCycPoint];
          if diff > maxCyc then
          begin
            maxCyc := diff;
            OldCycPoint := 0;
            NewCycPoint := 1;
            PointsOnCycle := 1;
            continue;
          end else if diff = maxCyc then
            if PointsOnCycle < MinCycPoints then
            begin
              PointsOnCycle += 1;
              OldCycPoint := NewCycPoint;
            end else begin
              T.period := maxCyc;
              break;
            end;
          NewCycPoint += 1;
        end;
      CycPoints += 1;
      dist := le;
    end;
    DerivateParser.Z := FormulaParser.IterateZ;
    inc(i);
  end;
  T.iterations := i;
  SetLength(T.Z,T.iterations);
  SetLength(T.LyaSum, T.iterations);
  SetLength(T.LyaPartSums, T.iterations);
end;

{###########################################################################
 |                              Properties                                 |
 ###########################################################################}

procedure TFractal.SetColRepeat(X: Integer);
begin
  if (X <> colRepeat) then
  begin
    colRepeat := X;
    colorchanged := true;
  end;
end;

procedure TFractal.SetOutercolormode(mode: shortint);
begin
  if mode <> Outercolmode then
  begin
    colorchanged:=true;
    case mode of
      0: if LastOuterValid <> mode then changed := true;
    end;
    OuterColormode := mode;
  end;
end;

procedure TFractal.SetInnercolormode(mode: shortint);
begin
  if mode <> Innercolmode then
  begin
    colorchanged := true;
    case mode of
      3,4: if (LastInnerValid <> 3) and (LastInnerValid <> 4)
        then changed := true;
    end;
    InnerColormode := mode;
  end;
end;

procedure TFractal.SetMaxIter(MI: integer);
begin
  if (MaxIter <> MI) then
  begin
    MaxIter := MI;
    SetLength(CycleArray, MaxIter+1);
    SetLength(CycArray, MaxIter+1);
    SetLength(Orbit, MaxIter+1);
    Changed := true;
  end;
end;

procedure TFractal.SetLyaUpperBorder(border: extended);
begin
  if border <> lyaUpperBorder then
  begin
    lyaUpperBorder := border;
    changed := true;
  end;
end;

procedure TFractal.SetLyapunovDwell(D: Double);
begin
  if D <> LyaDwell then
  begin
    LyaDwell := D;
    colorchanged := true;
  end;
end;

procedure TFractal.SetLyaLowerBorder(border: extended);
begin
  if border <> lyaLowerBorder then
  begin
    lyaLowerBorder := border;
    changed := true;
  end;
end;

function TFractal.GetLyapunov(X, Y: Integer): Double;
begin
  if (X >= 0) and (X < SizeX) and (Y >= 0) and (Y < SizeY) then
     Result:=data[Y,X].le
  else Result := 0;
end;

function TFractal.GetIterated(X, Y: integer): integer;
begin
  if (X >= 0) and (X < SizeX) and (Y >= 0) and (Y < SizeY) then
     Result:=data[Y,X].i
  else Result := 0;
end;

procedure TFractal.SetOuterColorDwellLimit(X: Integer);
begin
     if OuterColorDwellLimit<>X then
     begin
          OuterColorDwellLimit:=X;
          colorchanged:=true;
     end;
end;

procedure TFractal.SetInnerColortable(X: Integer; T: TPixel);
//ändert eine Farbe in der Grenzzyklentabelle
begin
  if InnerColorTable[X] <> T then
  begin
    //Alpha-Wert setzen
    InnerColorTable[X] := $FF000000 or T;
    Colorchanged := true;
  end;
end;

function TFractal.GetInnerColortable(X: Integer): TPixel;
//gibt eine Farbe aus der Grenzzyklentabelle zurück
begin
  Result := InnerColorTable[X];
end;

procedure TFractal.SetMinCycPoints(x: integer);
//setzt die Mindestanzahl von Prüfpunkten
begin
  if x <> minCycPoints then
  begin
    minCycPoints := x;
    Changed := true;
    Colorchanged := true;
  end;
end;

function TFractal.GetCalculated(X, Y: Integer): boolean;
begin
  Result := data[Y,X].calculated;
end;

function TFractal.GetOuterColortable(X: Integer): TPixel;
begin
  Result := OuterColorTable[X];
end;

procedure TFractal.SetOuterColortable(x: Integer; T: TPixel);
begin
  if OuterColorTable[X] <> T then
  begin
    OuterColorTable[X] := $FF000000 or T;
    Colorchanged := true;
  end;
end;

function TFractal.GetPeriod(X, Y: Integer): integer;
begin
  if (X >= 0) and (X < SizeX) and (Y >= 0) and (Y < SizeY) then
    Result := data[Y, X].a
  else Result := 0;
end;

procedure TFractal.SetcalculatingMode(c: integer);
begin
  if calculatingmode <> c then
  begin
    calculatingmode := c;
    changed := true;
  end;
end;

function TFractal.GetFormula: string;
begin
     Result:=FormulaParser.ParsedString;
end;

function TFractal.GetCriticalPoint: string;
begin
     Result:=CriticalParser.ParsedString;
end;

function TFractal.GetDerivateFormula: string;
begin
     Result:=DerivateParser.ParsedString;
end;

function TFractal.GetDerivate2Formula: string;
begin
     Result:=Derivate2Parser.ParsedString;
end;

function TFractal.GetDEMFormula: string;
begin
     Result:=DEMParser.ParsedString;
end;

function TFractal.GetDistance(X, Y: Integer): Double;
begin
  if (X >= 0) and (X < SizeX) and (Y >= 0) and (Y < SizeY) then
     Result:=data[Y,X].di
  else Result := 0;
end;

end.

